/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * COPSE Client Application
 * Classe:  COPSEClientProjectManager
\******************************************************************************/

import java.io.*;
import java.util.*;
import java.net.*;


import COPSE.*;
import COPSE.CLIENT.*;

import FILE.*;
import PRODUCT.*;


public class COPSEClientProjectManager
{
    COPSEClient         pClient;

    int                 mUserId;
    String              mCOPSEServerHost;
    int                 mCOPSEServerPort;


    COPSEClientProductManager       pProductManager;
    COPSEClientUsersManager         pUsersManager;
    COPSEClientProcessManager       pProcessManager;


    Vector                          pClientManagers;


    Vector                          pManagerListeners;

/*
    CFileController     pProductController;
    CProduct            pProduct;

    CFileController     pUsersController;
    CFile               pUsers;
*/


    public COPSEClientProjectManager ( COPSEClient client, int userId, String host, int port )
    {
        pClient = client;
        mUserId = userId;
        mCOPSEServerHost = host;
        mCOPSEServerPort = port;

        pProductManager = new COPSEClientProductManager( this );
        pUsersManager   = new COPSEClientUsersManager( this );

        // Ask for socket to the process server
        try {
            CEvent event = new CEvent( CEvent.EvCOPSE_ProcessManager_ServerSocket );
            client.GetConnector().Send( event );
        } catch ( Exception e ) {};

//        pProcessManager = COPSEClientProcessManager.New( this );

        pClientManagers = new Vector();
        pManagerListeners = new Vector();

    }

    public COPSEClient  GetClient()                     { return pClient; }
    public int          GetUserId()                     { return mUserId; }
    public String       GetProjectServerHost()          { return mCOPSEServerHost; }
    public int          GetProjectServerPort()          { return mCOPSEServerPort; }


    public COPSEClientProductManager    GetProductManager()     { return pProductManager; }
    public COPSEClientUsersManager      GetUsersManager()       { return pUsersManager; }
    public COPSEClientProcessManager    GetProcessManager()     { return pProcessManager; }


    public void addManagerListener( COPSEClientManagerListener listener )
    {
        pManagerListeners.add( listener );
    }

    public void removeManagerListener( COPSEClientManagerListener listener )
    {
        pManagerListeners.remove( listener );
    }

    public void StartProcessManager( String host, int port )
    {
        try {
            pProcessManager = COPSEClientProcessManager.New( this, host, port, pClient.GetConnector() );
            pProcessManager.start();
        } catch ( Exception e ) {
            System.err.println( "[DEBUG] COPSEClientProjectManager - Error on starting ProcessManager" );
            e.printStackTrace( System.err );
            return;
        };
        System.err.println( "[DEBUG] COPSEClientProjectManager - ClientProcessManager running..." );
    }


    public CClientManager FindClient( int serverPort, int connectionId )
    {
        if( pProcessManager != null )
            if( (pProcessManager.GetServerPort() == serverPort) &&
                (pProcessManager.GetConnectionId() == connectionId) )
            {
                return pProcessManager;
            }

        if( !pClientManagers.isEmpty() )
        {
            Enumeration e = pClientManagers.elements();
            while( e.hasMoreElements() )
                try{
                    CClientManager client = (CClientManager) e.nextElement();
                    if( (client.GetServerPort() == serverPort) &&
                        (client.GetConnectionId() == connectionId) )
                    {
                        return client;
                    }
                } catch( Exception exc ) {};
        }

        return null;
    }

    public void ExitAllClients()
    {
        // Sending CLOSE events to servers...
        System.err.println( "[DEBUG] ProjectManager - Sending close events to servers..." );
        if( !pClientManagers.isEmpty() )
        {
            Enumeration e = pClientManagers.elements();
            while( e.hasMoreElements() )
                try{
                    CClientManager client = (CClientManager) e.nextElement();
                    client.Exit();
                } catch( Exception exc ) {};
        }

        if( pProcessManager != null )
            pProcessManager.Exit();

        // Waiting clients finishing...
        System.err.println( "[DEBUG] ProjectManager - Waiting for clients finishing..." );
        if( !pClientManagers.isEmpty() )
        {
            Enumeration e = pClientManagers.elements();
            while( e.hasMoreElements() )
                try{
                    System.err.println( "[DEBUG] ProjectManager - Joining a client..." );
                    CClientManager client = (CClientManager) e.nextElement();
                    client.join();
                    //if( client.isAlive() )          client.stop();
                } catch( Exception exc ) {};
        }

        if( pProcessManager != null )
            try {
                System.err.println( "[DEBUG] ProjectManager - Joining ProcessManager" );
                pProcessManager.join();
                System.err.println( "[DEBUG] ProjectManager - ProcessManager finished!" );
                //if( pProcessManager.isAlive() )     pProcessManager.stop();
            }catch( Exception e )
            {
                System.err.println( "[DEBUG] ProjectManager - Error on joining ProcessManager..." );
            }

        System.err.println( "[DEBUG] ProjectManager - All clients finished!" );

    }


    public void Close()
    {
        pUsersManager.Close();
        pProductManager.Close();
    }

}

