/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE UI_Interface
 * PACOTE INTERFACE
 * DATA DE CRIACAO 15 / 11 / 2000
 */

package BW.Interface;

import BW.kernel.*;
import BW.control.FC_CL_Context;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;


/**
 * A classe UI_Interface age como controladora da apresentacao
 * das informacoes de awareness para o usuario. Ela mantem as
 * referencias para os elementos de interface que realmente
 * implementam a interface, e faz as chamadas as demais camadas
 * do framework BW, em busca das informacoes de awareness.
 * @see UI_GUIElement
 * @author Manuele Kirsch Pinheiro
 * @version 1.0
 */
public class UI_Interface {

	////////////////////////// ATRIBUTOS

        /** Relacao dos elementos de interface com os tipos de
         * eventos que eles apresentam, no formato (UI_GUIElement,Vector de eventos)
         */
	Hashtable UI_elements;
	
	/** Membro ativo no momento */
	BW_Member act_member;
	
	/** Fachada que permite o acesso ao restante do
	 * framework BW, feito por meio do pacote control.
	 */
	FC_CL_Context control;
	
	
	//////////////////////////// CONSTRUTOR //////////////////////
	
	public UI_Interface () {
		UI_elements = new Hashtable();
		act_member = null;
		control = null;
	}

		
        /** Finaliza as operacoes */
	public void terminate () {	
	
		System.out.println ("[DBG]UI_Interface.terminate: finalizing activities");			
		Enumeration elem = UI_elements.keys();
		while (elem.hasMoreElements())
			((UI_GUIElement)elem.nextElement()).Hide();
			
	        control.terminate ();			
	}
	
	
	////////////////////////////  METODOS /////////////////////////
	
	/** Sinaliza o login de um usuario
	 * @param BW_Member memb	membro ativo no sistema
	 * @see #setMember
	 */
	public void memberLogin (BW_Member memb) {
	     try {
		control.memberLogin(memb);
	     } catch (NullPointerException e) {
	     }		
	}	
	
	/** Sinaliza o logout de um usuario do sistema
	 * @param BW_Member memb	membro saindo do sistema
	 */
	public void memberLogout (BW_Member memb) {
	    try {
		control.memberLogout (memb);	
		
		//Se eh o membro ativo, desloga-o
		if (memb.getTypeId()==act_member.getTypeId() &&
			 memb.getObjId()==act_member.getObjId()) {
			act_member = null;
		}
	    } catch (NullPointerException e) {
	    	System.out.println ("[DBG]UI_Interface: Exception");
	    	e.printStackTrace();
	    }	
	}

	/** Sinaliza o logout do usuario ativo do sistema */
	public void memberLogout () {
	    try {
		control.memberLogout (act_member);	
		
		act_member = null;		
	    } catch (NullPointerException e) {
	    }	
	}
	

	/** Sinaliza o login de um usuario, que ficara de usuario
	 * ativo para a interface
	 * @see #act_member
	 * @param BW_Member memb	membro ativo da interface
	 */
	public void setMember (BW_Member memb) {
	    try {
		//verifica se havia um membro setado antes e faz o logout dele
		if (act_member != null) {
			control.memberLogout(act_member);
			act_member=null;
		}
	        //faz o login no membro
		BW_Member loaded = control.memberLogin (memb);		
		if (loaded!=null)
			act_member = loaded;
	        else
	        	act_member = memb;
		System.out.println ("[DBG]UI_Interface.setMember: login "+act_member.getLogin());	
	    } catch (NullPointerException e) {
		System.out.println ("[DBG]UI_Interface.setMember: Exception");
		e.printStackTrace();
	    }
	}
	
	
	/** Retorna as informacoes do membro ativo no momento
	 * @return BW_Member	informacoes do membro ativo
	 */
	public BW_Member getMember () {
		//return act_member;	
		return this.getMemberInfo(act_member);
	}

		
	/** Retorna as informacoes completas de um membro em especifico
	 * @param BW_Member memb	membro que se deseja buscar. Precisa
	 *				estar, no minimo, com o uid setado.
	 * @see BW.kernel.BW_Member
	 * @return BW_Member	informacoes completas sobre um membro
	 *			null se o membro nao existir
	 */	
	public BW_Member getMemberInfo (BW_Member memb) {
		BW_Member res = null;
	     	try {	
			res = control.getMemberInfo (memb);
		} catch (NullPointerException e) {
	      	}
		return res;
	}
	
	/** Altera as informacoes de um membro especifico.
	 * @param BW_Member	membro a ser alterado
	 */
	public void setMemberInfo (BW_Member memb) {
	    try {
		control.setMemberInfo (memb);		
		System.out.println ("[DBG]UI_Interface.setMemberInfo: seting: "+memb.getLogin());
	    } catch (NullPointerException e) {
	    }	
	}

		
	/** Retorna o grupo ativo no sistema
	 * @return BW_Group	grupo do sistema
	 */
	public BW_Group getGroup () {
		BW_Group group = null;
		try {
			group = control.getGroup();
		} catch (NullPointerException e) {
	    	}	
		return group;
	}
	
	
	/** Busca os eventos de interesse do usuario ativo no momento.
	 * @param UI_GUIElement gui	elemento de interface que esta
	 *				fazendo a chamada
	 * @return Vector	vetor de BW_Events com todos os eventos
	 *			de acordo com os profiles do usuario
	 * @see BW.kernel.BW_Event
	 * @see BW.kernel.BW_Member#profiles
	 */		 				
	public Vector getEvents (UI_GUIElement gui) {
	     Vector res = null;
	     Vector evts = null;
	     //System.out.println ("[DBG] UI_Interface.getEvents: begin ");
	
	     try {
	     	//busca os eventos
		evts = control.getEvents (act_member);
		//System.out.println ("[DBG] UI_Interface.getEvents: no. available events: "+evts.size());
		
	        //separa os eventos de interesse de gui
	        Vector evt_prot = (Vector) UI_elements.get (gui);	
		//System.out.println ("[DBG] UI_Interface.getEvents: evts_prot: "+evt_prot);
		
		res = new Vector();
		
		Enumeration e1 = evts.elements();
		while (e1.hasMoreElements()) {
			BW_Event evento = (BW_Event) e1.nextElement();
			String s = this.mountKey (evento);
			//System.out.println ("[DBG] UI_Interface.getEvents: analizing "+
			//			"("+evento.getTypeId()+","+evento.getObjId()+") = "+s);
			if (evt_prot.contains (s)) {
				res.addElement (evento);
				//System.out.println ("[DBG] UI_Interface.getEvents: reading "+evento);
			}
		}
	
	     } catch (NullPointerException npe) {
	
	     }
	     System.out.println ("[DBG]UI_Interface.getEvents: returning "+res.size()+" events");	
	     return res;	
	}
	
	
	/** Mensagem avisando sobre a necessidade de se apresentar
	 *  as informacoes de awareness para o usuario default.
	 */
	public void Notify () {
	      //alerta todos os elementos de interface sobre a
	      //necessidade de atualizacao	
	      Enumeration e = UI_elements.keys();
	      while (e.hasMoreElements()) {
	      	UI_GUIElement gui = (UI_GUIElement) e.nextElement();
	      	//System.out.println ("[DBG]UI_Interface.Notify: "+gui);
	      	gui.Notify();
	      }	
	}
	

	/** Registra um elemento de interface como responsavel pela
	 *  apresentacao de um tipo de evento
	 * @param UI_GUIElement gui_elem 	elemento de interface sendo registrado
	 * @param Vector evt_prot	vetor com prototipos de eventos, para os quais
	 *				gui_elem sera o responsavel pela apresentacao
	 */
	public void addGUIElement (UI_GUIElement gui_elem,Vector evt_prot) {		
		//processa o vetor de eventos
		Vector evts = new Vector();
		Enumeration enum = evt_prot.elements();
		while (enum.hasMoreElements()) {
			BW_Event e = (BW_Event) enum.nextElement();
			evts.addElement (this.mountKey (e));
		}	
		
		 //coloca o para UI_GUIElement,eventos na hash		
		UI_elements.put (gui_elem,evts);		
	}
	
	
	public String mountKey (BW_Event obj) {
		   Class classe = obj.getClass();
		   StringBuffer key = new StringBuffer();
		   //key.append ((obj.getClass().getName()).toUpperCase());
		   key.append ((obj.getClass().getName()));
		   key.append("-");
		   key.append(obj.getTypeId());	         	
		   return key.toString();
	}
	
	/** seta o objeto de conexao (fachada) com a camada de controle
	 * @param	FC_CL_Context cont	fachada para o pacote control
	 */	
	public void setFCCLContext (FC_CL_Context cont) {
		this.control = cont;
	}	
}