/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE UI_MultiInterface
 * PACOTE INTERFACE
 * DATA DE CRIACAO 30 / 07 / 2001
 */

package BW.Interface;

import BW.kernel.*;
import BW.Interface.UI_Interface;
import BW.control.FC_CL_Context;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;


/**
 * A classe UI_MultiInterface age como controladora da apresentacao
 * das informacoes de awareness para multiplos usuario. Ela nao trabalha
 * diretamente com os elementos de interface destes usuarios, mas
 * permite a busca por eventos de interesse de cada um destes usuarios
 * em separado (fazendo chamadas as demais camadas do framework BW),
 * deixando a apresentacao toda a cargo do gropware.
 * Esta estrutura permite que se possa colocar o framework BW no lado
 * servidor de um groupware, podendo atuar com varios clientes simultaneamente.
 * 
 * @see UI_Interface
 * @author Manuele Kirsch Pinheiro
 * @version 1.0
 */
public class UI_MultiInterface extends UI_Interface {

	////////////////////////// ATRIBUTOS
	
	/** Lista de user ativos atraves desta instancia da camada de interface,
	 *  organizada na forma de tuplas (login,BW_Member).
	 */
	Hashtable userlist;
	
	
	//////////////////////////// CONSTRUTOR //////////////////////
	
	public UI_MultiInterface () {
		super();
		userlist = new Hashtable();
	}

		
        /** Finaliza as operacoes */
	public void terminate () {	
	
		System.out.println ("[DBG]UI_MultiInterface.terminate: finalizing activities");			
		Enumeration elem = userlist.elements();
		while (elem.hasMoreElements()) {
			BW_Member memb = (BW_Member) elem.nextElement();
			System.out.println ("[DBG] UI_MultiInterface.terminate: login out user "+memb.getLogin());			
			control.memberLogout (memb);
		}
			
	        super.terminate();
	}
	
	
	////////////////////////////  METODOS /////////////////////////
	
	/** Sinaliza o login de um usuario
	 * @param BW_Member memb	membro ativo no sistema
	 * @see #setMember
	 */
	public void memberLogin (BW_Member memb) {
	     try {
		control.memberLogin(memb);
		this.addUser (memb);

	     } catch (NullPointerException e) {
	     }		
	}	
	

	/** Sinaliza o logout de um usuario do sistema
	 * @param BW_Member memb	membro saindo do sistema
	 */
	public void memberLogout (BW_Member memb) {
	    try {
		//Se eh o membro ativo, desloga-o usando metodo da classe mae
		BW_Member act = this.getMember();
		
		if (act != null && 
			memb.getTypeId()==act.getTypeId() && 
				memb.getObjId()==act.getObjId()) {
			super.memberLogout();
		}
		else  { //senao, faz o processamento localmente
			control.memberLogout (memb);	
		}

		//no final, remove da userlist
		this.removeUser (memb);

	    } catch (NullPointerException e) {
	    	System.out.println ("[DBG]UI_MultiInterface: Exception");
	    	e.printStackTrace();
	    }	
	}


	/** Sinaliza o logout do usuario ativo do sistema */
	public void memberLogout () {
	    try {
		BW_Member memb = this.getMember();
		super.memberLogout();	
		this.removeUser (memb);		
				
	    } catch (NullPointerException e) {
	    }	
	}
	

	/** Sinaliza o login de um usuario, que ficara de usuario
	 * ativo para a interface
	 * @see #act_member
	 * @param BW_Member memb	membro ativo da interface
	 */
	public void setMember (BW_Member memb) {
	    try {
		super.setMember(memb);
	 	this.addUser (memb);

	    } catch (NullPointerException e) {
		System.out.println ("[DBG]UI_MultiInterface.setMember: Exception");
		e.printStackTrace();
	    }
	}
	
	
	/** Busca os eventos de interesse do usuario passado como parametro
	 * @param BW_Member user	usuario para o qual se esta
	 *				fazendo a chamada
	 * @return Vector	vetor de BW_Events com todos os eventos
	 *			de acordo com os profiles do usuario
	 * @see BW.kernel.BW_Event
	 * @see BW.kernel.BW_Member#profiles
	 * @see BW.Interface.UI_Interface#getEvents
	 */		 				
	public Vector getEvents (BW_Member user) {
	     Vector evts = null; //eventos
	     Vector res = null;   //retorno
	     System.out.println ("[DBG] UI_MultiInterface.getEvents: begin ");
	
	     try {
	     	//busca os eventos
		System.out.println ("[DBG] UI_MultiInterface.getEvents: Membro :  " + user);
		evts = control.getEvents (user);
		System.out.println ("[DBG] UI_MultiInterface.getEvents: no. available events: "+evts.size());
		res = evts;
	     } catch (NullPointerException npe) {
		res = new Vector();
	     }

	     System.out.println ("[DBG] UI_MultiInterface.getEvents: returning "+res.size()+" events");
	     return res;	
	}



	////////////////////////////// METODOS SOBRE A USER LIST ////////////////////////////////////////
	
	/** retorna a lista de usuarios ativos para esta instancia da camada de interface 
	 *  @return	Hashtable	Hash com os usuarios ativos no formato (String login, BW_Member user) 
	 */
	public Hashtable getUserList () { 
		return this.userlist;
	}

	/** adiciona user na userlist */
	void addUser (BW_Member memb) {
		userlist.put (memb.getLogin(),memb);
	}

	/** remove user da userlist */
	void removeUser (BW_Member memb) {
		userlist.remove (memb.getLogin());		
	}
}
