/**
* UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
* AUTOR LUIZ ANGELO BARCHET ESTEFANEL
* CLASSE AwarenessProfileWindow
* PACOTE BW.UML
* DATA DE CRIACAO 16 / 01 / 2001
*/

package BW.UML;

import java.awt.*;
import java.awt.event.*;
import BW.typeIdDefault;
import BW.kernel.*;
import BW.Interface.*;
import java.util.*;
import COPSE.CEvent;

/**
* A classe AwarenessProfileWindow exibe os parametros de filtragem
* dos profiles do usuario, bem como permite modificar esses parametros 
* @author Luiz Angelo Barchet Estefanel
* @author Manuele Kirsch Pinheiro
* @version 1.0
*/
public class AwarenessProfileWindow extends Frame implements WindowListener, ItemListener, ActionListener {

   ////////////////////////// ATRIBUTOS
   
   /** variavel que contem os dados do membro
   */
   private BW_Member member;
   private BW_AwarenessProfile awarenessProfile;
   
   
   /** flag que indica se algum dos dados do usuario sofreram alteracao
   */
   private boolean isChanged = false;

   /** flag que indica se a janela esta' sendo usada para o personal profile
   *   ou para o papel
   */
   private boolean isPersonal;

   /** UI_Interface
   */
   private UI_Interface Interface;

   /** lista de eventos que podem ser selecionados
   *   events[0] -> BW_Event
   *   events[1] -> Checkbox
   */
   private Vector events[] = new Vector[2];

   /** vectors utilizados para a filtragem por periodo */
   Vector from;
   Vector to;   

     
   /** Variaveis da Interface grafica
   */

      private Panel background;
      private Panel North;
      private Panel Center;
	private Panel ckBox;
      private Panel South;
	private Panel timeBox;
      private Label Username;
      private Button okButton;
      private Button cancelButton;


   ////////////////////////// CONSTRUTOR  
   
   /**
	 * Construtor
	 * @param UI_Interface interface referencia a janela principal
	 * @param BW_Member member referencia ao registro do usuario
	 * @param boolean purpose se true=PersonalProfile, se false=PersonalPaperProfile
	 */

   public AwarenessProfileWindow (UI_Interface Interface, BW_Member member, boolean purpose) {
      super("Profile Edition");
      this.Interface = Interface;
      this.member = member;
      this.isPersonal = purpose;

      resetAllEvents();

      this.setSize(400, 400);
      addWindowListener (this);
      background = new Panel();
      add(background);
      background.setLayout(new BorderLayout());
      Center = new Panel();
	ckBox=new Panel(new GridLayout(5,2));
	timeBox = new Panel (new GridLayout(6,1));
	Center.add(ckBox);
	Center.add(timeBox);

	
      
      North = new Panel();
      South = new Panel();

      background.add("North",North);
      background.add("Center", Center);
      background.add("South",South);

      try {

      	if (isPersonal==true) {
        	Username = new Label (member.getName() + "'s Personal Profile");
      	} else {
         	Username = new Label ("Profile Options");
      	}
      } catch (NullPointerException npe) {
     		Username = new Label ("UNKNOWN");
     		npe.printStackTrace();
      }

	North.add(Username); 

      	for (int i=0; i<events[0].size();++i)
      	{
         	ckBox.add((Checkbox)events[1].elementAt(i));
         	((Checkbox)events[1].elementAt(i)).addItemListener(this);
      	}
      
	
	Label fromLabel = new Label("From:");
	from = new Vector();
	to = new Vector();
	for (int i=0; i<6; ++i) {
           from.addElement(new Choice());
	   to.addElement(new Choice());
	}
        for (int i=1; i<=31; ++i) {
	   ((Choice)from.elementAt(0)).add(String.valueOf(i));
	   ((Choice)to.elementAt(0)).add(String.valueOf(i));
	}
	for (int i=1; i<=12; ++i) {
           ((Choice)from.elementAt(1)).add(String.valueOf(i));
           ((Choice)to.elementAt(1)).add(String.valueOf(i));
        }
        for (int i=1999; i<=2010; ++i) {
           ((Choice)from.elementAt(2)).add(String.valueOf(i));
           ((Choice)to.elementAt(2)).add(String.valueOf(i));
        }
        for (int i=0; i<=23; ++i) {
           ((Choice)from.elementAt(3)).add(String.valueOf(i));
           ((Choice)to.elementAt(3)).add(String.valueOf(i));
        }
        for (int i=0; i<=59; ++i) {
           ((Choice)from.elementAt(4)).add(String.valueOf(i));
           ((Choice)to.elementAt(4)).add(String.valueOf(i));
        }
        for (int i=0; i<=59; ++i) {
           ((Choice)from.elementAt(5)).add(String.valueOf(i));
           ((Choice)to.elementAt(5)).add(String.valueOf(i));
        }

	Panel fromData = new Panel(new GridLayout(1,6));
	fromData.add(new Label("Date:"));
	fromData.add((Choice)from.elementAt(0));
	fromData.add(new Label("/"));
	fromData.add((Choice)from.elementAt(1));
	fromData.add(new Label("/"));
	fromData.add((Choice)from.elementAt(2));

	Panel fromHour = new Panel(new GridLayout(1,6));
	fromHour.add(new Label("Hour:"));
	fromHour.add((Choice)from.elementAt(3));
	fromHour.add(new Label(":"));
	fromHour.add((Choice)from.elementAt(4));
	fromHour.add(new Label(":"));
	fromHour.add((Choice)from.elementAt(5));

	Label toLabel = new Label("To:");
	
	Panel toData = new Panel(new GridLayout(1,6));
	toData.add(new Label("Date:"));
	toData.add((Choice)to.elementAt(0));
	toData.add(new Label("/"));
	toData.add((Choice)to.elementAt(1));
	toData.add(new Label("/"));
	toData.add((Choice)to.elementAt(2));

	Panel toHour = new Panel(new GridLayout(1,6));
	toHour.add(new Label("Hour:"));
	toHour.add((Choice)to.elementAt(3));
	toHour.add(new Label(":"));
	toHour.add((Choice)to.elementAt(4));
	toHour.add(new Label(":"));
	toHour.add((Choice)to.elementAt(5));

	timeBox.add(fromLabel);
	timeBox.add(fromData);
	timeBox.add(fromHour);
	timeBox.add(toLabel);
	timeBox.add(toData);
	timeBox.add(toHour);

	for (int i=0; i<6; ++i) {
        	((Choice)from.elementAt(i)).addItemListener(this);
        	((Choice)to.elementAt(i)).addItemListener(this);
        	
		//loadMember();
	}

      loadMember();

      okButton = new Button ("Ok");
      okButton.addActionListener(this);
      South.add(okButton);
      cancelButton = new Button ("Cancel");
      cancelButton.addActionListener(this);
      South.add(cancelButton);
      setVisible(true);
 
      /* nao sei porque, mas um sleep impede que o TextListener "escute"
       * a definicao do ambiente grafico */
      try {
	 Thread.sleep(200);
      } catch (InterruptedException ex) {
      }
 
      isChanged = false;
      
   }


   ////////////////////////  METODOS

   /** metodo loadMember preenche os campos com os dados atuais do usuario
    */
   public void loadMember () {

    try {

      	awarenessProfile = member.getAwarenessProfile();
      	BW_Paper paper = null, paper2 = null;
      	BW_Profile profile;
			
	BW_TimeInterval timeInterval;
	Hashtable eventList;

      	if (isPersonal)
      	{
      		profile = awarenessProfile.getPersonalProfile(0);
      	}
      	else
      	{
    	 	paper = member.getActivePaper();
	 	paper2 = member.getPaper(paper.getName());
    	 	if (paper2 != null)
    	 	{	 	
    	 		profile = awarenessProfile.getPersonalPaperProfile(paper2);
    	 	} else {
    	       		System.out.println("there's no active paper");
    	       		//profile = new BW_Profile (typeIdDefault.BW_PROFILE,typeIdDefault.UNSET);
    	       		profile = new BW_Profile (typeIdDefault.BW_PROFILE,UML_BW_Mediator.GetNextObjId());
         	}
      	}

      	try {
         	eventList = profile.getEvents();
	 	for (int i=0; i<events[0].size(); ++i)
      	 	{
            		if (eventList.containsKey(mountKey((BW_Event)events[0].elementAt(i))))
               			((Checkbox)events[1].elementAt(i)).setState(true);
            		else
	       			((Checkbox)events[1].elementAt(i)).setState(false);
	      	}
       	} catch (NullPointerException ex) {
       		System.out.println ("[DBG]AwarenessProfileWindow.loadMember: no events");       		
       	}

  	Calendar c1;
  	Calendar c2;

       	timeInterval = profile.getInterval();
    	c1 = timeInterval.getBegin();
    	c2 = timeInterval.getEnd();

    	System.out.println ("[DBG]AwarenessProfileWindow: original interval: "+timeInterval);

	((Choice)from.elementAt(0)).select(String.valueOf(c1.get(Calendar.DAY_OF_MONTH)));
	((Choice)from.elementAt(1)).select(String.valueOf(c1.get(Calendar.MONTH)+1));
	((Choice)from.elementAt(2)).select(String.valueOf(c1.get(Calendar.YEAR)));
	((Choice)from.elementAt(3)).select(String.valueOf(c1.get(Calendar.HOUR_OF_DAY)));
	((Choice)from.elementAt(4)).select(String.valueOf(c1.get(Calendar.MINUTE)));
	((Choice)from.elementAt(5)).select(String.valueOf(c1.get(Calendar.SECOND)));

	((Choice)to.elementAt(0)).select(String.valueOf(c2.get(Calendar.DAY_OF_MONTH)));
	((Choice)to.elementAt(1)).select(String.valueOf(c2.get(Calendar.MONTH)+1));
	((Choice)to.elementAt(2)).select(String.valueOf(c2.get(Calendar.YEAR)));
	((Choice)to.elementAt(3)).select(String.valueOf(c2.get(Calendar.HOUR_OF_DAY)));
	((Choice)to.elementAt(4)).select(String.valueOf(c2.get(Calendar.MINUTE)));
	((Choice)to.elementAt(5)).select(String.valueOf(c2.get(Calendar.SECOND)));
	
     } catch (NullPointerException npe) {
     	System.out.println ("[DBG]AwarenessProfileWindow: ERROR");
     	npe.printStackTrace();
     }	
   
   }

   /** monta a chave usada na hash que mantem os eventos de interesse
   * A chave e uma string formada pelo nome da
   * classe do evento mais o sinal de "-" (hifem), mais
   * o typeId da classe;
   * e o valor e o proprio typeId do evento
   */
   public String mountKey (BW_Event obj) {
      Class classe = obj.getClass();
      StringBuffer key = new StringBuffer();
      //key.append ((obj.getClass().getName()).toUpperCase());
      key.append ((obj.getClass().getName()));
      key.append("-");
      key.append(obj.getTypeId());
      return key.toString();
   }


	/**
	 * reseta os vetores e variaveis, para a inicializacao da janela
	 */
   public void resetAllEvents () {
      events[0] = new Vector(); // contem os objetos BW_Event
      events[1] = new Vector(); // contem os Checkbox
      Vector prototypes = UML_BW_Mediator.getPrototypes(); 
      for (int i=0;i<prototypes.size();++i) {
         events[0].addElement(prototypes.elementAt(i));
         events[1].addElement(new Checkbox(((BW_Event)prototypes.elementAt(i)).getName()));
      }
   }


	/**
	 * registra os parametros marcados nos profiles do usuario
	 */

   public void changeProfile () {

      BW_Paper paper = null, paper2=null;
      BW_Profile profile; 
	
      BW_TimeInterval timeInterval;
      Hashtable eventList;
      if (isPersonal)
      {
      	 profile = awarenessProfile.getPersonalProfile(0);   
	}
      else
	   {
	    	 paper = member.getActivePaper();
		 paper2 = member.getPaper(paper.getName());
	    	 if (paper2 != null)
	    	 {
	    	 	profile = awarenessProfile.getPersonalPaperProfile(paper2);
			if (profile == null)
				//profile = new BW_Profile (typeIdDefault.BW_PROFILE, typeIdDefault.UNSET);
				profile = new BW_Profile (typeIdDefault.BW_PROFILE, UML_BW_Mediator.GetNextObjId());
		 } else {
		 	///////////////////// CUIDAR //////////////////////////
	    	   System.out.println("there's no active paper");
		   paper = new BW_Paper();
	    	   profile = new BW_Profile (typeIdDefault.BW_PROFILE, typeIdDefault.UNSET); 
	    	 }
	    }
	
      eventList = new Hashtable();
	 		
      for (int i=0; i<events[0].size(); ++i)
      {
      	 if (((Checkbox)events[1].elementAt(i)).getState())
	 {
      	 	  eventList.put(mountKey((BW_Event)events[0].elementAt(i)),(BW_Event)events[0].elementAt(i));
	 }
      }
			profile.setEvents(eventList);
			
			timeInterval = new BW_TimeInterval();
			      
			Calendar c1 = Calendar.getInstance ();
			Calendar c2 = Calendar.getInstance ();
			
			int fromday = Integer.parseInt(((Choice)from.elementAt(0)).getSelectedItem());
			
			int frommonth = Integer.parseInt(((Choice)from.elementAt(1)).getSelectedItem())-1;
			int fromyear = Integer.parseInt(((Choice)from.elementAt(2)).getSelectedItem());
			int fromhour = Integer.parseInt(((Choice)from.elementAt(3)).getSelectedItem());
			int fromminute = Integer.parseInt(((Choice)from.elementAt(4)).getSelectedItem());
			int fromsecond = Integer.parseInt(((Choice)from.elementAt(5)).getSelectedItem());
			c1.set(fromyear,frommonth,fromday,fromhour,fromminute,fromsecond);
			
			int today = Integer.parseInt(((Choice)to.elementAt(0)).getSelectedItem());
			int tomonth = Integer.parseInt(((Choice)to.elementAt(1)).getSelectedItem())-1;
			int toyear = Integer.parseInt(((Choice)to.elementAt(2)).getSelectedItem());
			int tohour = Integer.parseInt(((Choice)to.elementAt(3)).getSelectedItem());
			int tominute = Integer.parseInt(((Choice)to.elementAt(4)).getSelectedItem());
			int tosecond = Integer.parseInt(((Choice)to.elementAt(5)).getSelectedItem());
			c2.set(toyear,tomonth,today,tohour,tominute,tosecond);
			
			timeInterval.setInterval(c1,c2);
			
			profile.setInterval(timeInterval);
		
		
			if (isPersonal)
			{
				Vector profs = new Vector();
				profs.addElement(profile);
				awarenessProfile.setPersonalProfile(profs);
				awarenessProfile.setInterval(profile.getInterval());
				member.setAwarenessProfile(awarenessProfile);
			} else {
				member.setPersonalPaperProfile(paper2,profile);
			}
         
      Interface.setMemberInfo(member);
      //System.out.println ("[DEBUG]AwarenessProfileWindow.changeProfile: NOTIFY");
      //Interface.Notify();

   }


	/**
	 * seta o Checkbox do evento indicado para o estado passado como parametro
	 * @param BW_Event target	evento que sera' selecionado/removido da lista de filtragem
	 * @param boolean state true=ativa false=desativa o Checkbox
	 * @see Checkbox
	 */
   public void setState (BW_Event target, boolean state) {
      for (int i=0; i<events[0].size(); ++i) {
         if ((BW_Event)events[0].elementAt(i)==target)
            ((Checkbox)events[1].elementAt(i)).setState(state);
      }
   }


	/**
	 * retorna o estado do Checkbox relativo ao evento
	 * @param BW_Event target	evento
	 * @return boolean estado do Checkbox
	 */

   public boolean getState (BW_Event target) {
      for (int i=0; i<events[0].size(); ++i) {
         if ((BW_Event)events[0].elementAt(i)==target)
            return ((Checkbox)events[1].elementAt(i)).getState();
      }
      return false;
   }


	/**
	 * retorna o evento relativo a um Checkbox
	 * @param Checkbox target	Checkbox do ambiente grafico
	 * @return BW_Event evento relativo ao Checkbox
	 */

   public BW_Event getEvent (Checkbox target) {
      for (int i=0; i<events[1].size(); ++i) {
         if ((Checkbox)events[1].elementAt(i)==target)
            return (BW_Event)events[0].elementAt(i);
      }
      return null;
   }

   private void okButton () {
      if (isChanged==true)
	    changeProfile();
      this.setVisible(false);
      dispose();
   }

   private void cancelButton () {
      this.setVisible(false);
      dispose();
   }


	//////// metodo do ItemListener

   public void itemStateChanged(ItemEvent e) {
      this.isChanged = true;
   }


	///////// metodo do ActionListener

   public void actionPerformed (ActionEvent e) {
      if ((e.getSource())==okButton)
	 okButton();
      else
	 if ((e.getSource())==cancelButton)
            cancelButton();
   }


	////////// metodos do WindowListener

   public void windowClosing (WindowEvent evt){
      this.setVisible(false);
      dispose();
   }



   public void windowClosed (WindowEvent evt) {
   }

   public void windowOpened (WindowEvent evt) {
   }

   public void windowIconified (WindowEvent evt) {
   }

   public void windowActivated (WindowEvent evt) {
   }

   public void windowDeactivated (WindowEvent evt) {
   }

   public void windowDeiconified (WindowEvent evt) {
   }

   public void actionPerformed (WindowEvent evt) {
   }

}