/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORES LUIZ ANGELO B. ESTEFANEL E MANUELE KIRSCH PINHEIRO
 * Classe ObjIdServer
 * PACOTE BW.UML.ObjId
 * DATA DE CRIACAO 28 / 01 / 2001
 * DATA DE ALTERACAO 30 / 05 / 2001
 */
package BW.UML.ObjId;

import BW.storage.ST_DatabaseImplementor;
import BW.storage.ST_Setup;
import BW.typeIdDefault;

import java.net.*;
import java.io.*;

/**
 * Classe ObjIdServer e um servidor de identificadores
 * de objetos para uso do framework para awareness BW,
 * quando combinado ao groupware CUTE (construido com
 * o framework COPSE).
 * Alteracoes feitas permitem o uso com outras aplicacoes,
 * nao so o COPSE/CUTE.
 * COPSE e CUTE sao de autoria de Marcio Dias, pela
 * COPPE/UFRJ, 1998.
 * @author Manuele Kirsch Pinheiro
 * @author Luiz Angelo B. Estefanel
 * @version 1.2
 */
public class ObjIdServer extends Thread {
	
	DatagramSocket socket;
	DatagramPacket packet;
	byte[] buf;
	InetAddress addressToSend;
	int portToSend;

		
	/** contador de ocorrencias - indica quantos
	 * identificadores foram fornecidos desde a
	 * ultima atualizacao do ST_Setup
	 */
	public static int cont = 0;
	
	/** numero de atendimentos a serem feitos, antes
	 * que a base de dados tenha o valor do ultimo
	 * identificador de objeto fornecido atualizado
	 */
	public static final int MAX = 1;
	
	
	/** proximo identificador de objeto disponivel */
	public static int lastObjId = typeIdDefault.OBJID;
	
	/** objeto de setup, onde fica armazenado o ultimo
	 * identificador de objeto utilizado */
	public static ST_Setup setup;
	
	
	/** acesso ao meio de armazenamento */
	public static ST_DatabaseImplementor db;
	
	
	/** construtor para o atendimento de uma chamada */
	public ObjIdServer (InetAddress addressToSend, int portToSend) {
		this.addressToSend = addressToSend;
		this.portToSend = portToSend;
	}
	
	
	/** construtor para a primeira chamada 
	 * @param	String server	servidor da base de dados
	 */
	public ObjIdServer (String server)	
	{
		//inicializa o acesso a base
		ObjIdServer.db = new ST_DatabaseImplementor(server);
		//le o objeto ST_Setup da base
		ObjIdServer.setup = db.read(new ST_Setup());
		if (ObjIdServer.setup == null) {
			System.out.println ("creating Setup");
			ObjIdServer.setup = new ST_Setup();
			ObjIdServer.setup.setServer (server);
			if (server.equals("localhost"))
				setup.setIsServer (true);
			else
				setup.setIsServer (false);
		}	
		
		//inicializa o lastobjid
		ObjIdServer.lastObjId = ObjIdServer.setup.getLastObjId();
	}
	
	
/** construtor alternativo, aceita como parametros alem do
	* host servidor, o nome do usuario e da base de dados a ser
	* utilizada.
	* @param	String server	nome do host
	* @param String dbname nome da base de dados
	* @param String user   nome do usuario 
	* @param String passwd	password do usuario
	*/	
	public ObjIdServer (String server, String dbname, String user, String passwd)	
	{
		//inicializa o acesso a base
		ObjIdServer.db = new ST_DatabaseImplementor(server,dbname,user,passwd);
		//le o objeto ST_Setup da base
		ObjIdServer.setup = db.read(new ST_Setup());
		if (ObjIdServer.setup == null) {
			System.out.println ("creating Setup");
			ObjIdServer.setup = new ST_Setup();
			ObjIdServer.setup.setServer (server);
			if (server.equals("localhost"))
				setup.setIsServer (true);
			else
				setup.setIsServer (false);
		}	
		
		//inicializa o lastobjid
		ObjIdServer.lastObjId = ObjIdServer.setup.getLastObjId();
	}
	
	
	/** comeca a ouvir a porta indicada */
	public void init (int port) {
		
		try {
			socket = new DatagramSocket(port);
			buf = new byte[32];
			packet = new DatagramPacket(buf, buf.length);
			// le setup, lastid
			while (true)
			//ReadEnd read = new ReadEnd();
			//read.start();
			//while (!read.isEnd())
			{       //System.out.println("readEnd "+read.isEnd());
				buf = new byte[32];
				System.out.println("esperando");
				socket.receive(packet);
				ObjIdServer threadOID = new ObjIdServer (packet.getAddress(), packet.getPort());
				threadOID.start();
			}
		} catch (SocketException ex) {
			System.out.println("Porta nao disponivel");
		} catch (IOException ex) {
			System.out.println("erro na leitura");
		}
	}
	
	
	/** atendimento da chamada */
	public void run () {
		
		Integer oid = this.nextObjId ();
		try {
			socket = new DatagramSocket();
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			ObjectOutputStream objout = new ObjectOutputStream(bout);
			objout.writeObject(oid);
			objout.flush();
			objout.close();
			byte[] sendBuf = bout.toByteArray();
			//System.out.println(sendBuf);
			DatagramPacket sendPacket = new DatagramPacket (sendBuf, sendBuf.length, addressToSend, portToSend);
			//System.out.println(addressToSend + "," + portToSend);
			socket.send(sendPacket);
			//System.out.println("impossivel enviar dados");
		} catch (IOException ex) {
			System.out.println("erro de IO");
		}
	}
	
	
	/** fornece o proximo identificador de objeto */
	public synchronized Integer nextObjId () {
		ObjIdServer.cont++;
		Integer ID = new Integer (ObjIdServer.lastObjId++);
		
		if (ObjIdServer.cont > ObjIdServer.MAX)
			this.update();
			
		return ID;
	}
	
	
	/** faz a atualizacao do ultimo identificador utilizado na base de dados */
	public synchronized void update () {
		ObjIdServer.setup.setLastObjId(ObjIdServer.lastObjId);
		ObjIdServer.db.write (ObjIdServer.setup);
		ObjIdServer.cont=0;
	}
	
}
		
		