/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe UML_SETUP
 * PACOTE BW.UML
 * DATA DE CRIACAO 27 / 01 / 2001
 * DATA DE ALTERACAO 17 / 05 / 2001
 */
package BW.UML;

//import BW.Interface.*;
import BW.control.*;
import BW.storage.*;
import BW.kernel.*;
import BW.*;

//import COPSE.CEvent;

import java.util.*;
import java.net.*;

/**
 * Classe UML_Setup faz o setup
 * da base de dados para o funcionamento
 * do COPSE/CUTE com o framework BW.
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class UML_Setup {


	/** cria um membro default */
	public static BW_Member makeDefaultMember (BW_Paper paper) {
		BW_Member member = new BW_Member (typeIdDefault.BW_MEMBER,UML_BW_Mediator.GetNextObjId());
		member.setLogin ("CUTE");
		member.setName ("Usuario default do COPSE/CUTE");
		
		BW_Profile personal_profile = UML_BW_Mediator.makeProfile();
		personal_profile.setTypeId (typeIdDefault.BW_PERSONALPROFILE);
		member.insertPersonalProfile (personal_profile);
		
		BW_Profile personal_paper = UML_BW_Mediator.makePersonalPaperProfile();
		personal_paper.setTypeId (typeIdDefault.BW_PERSONALPAPERPROFILE);
		member.insertPaper (paper,personal_paper);
		
		BW_AwarenessProfile awareness_profile = member.getAwarenessProfile();
		awareness_profile.setObjId(UML_BW_Mediator.GetNextObjId());
		member.setAwarenessProfile (awareness_profile);
		
	        return member;
	}
	
	/** cria um papel default */
	public static BW_Paper makeDefaultPaper () {
		BW_Paper paper = new BW_Paper (UML_BW_Mediator.PAPERTYPEID,UML_BW_Mediator.PAPEROBJID);
		paper.setName ("member");
		paper.setDescription ("Partipante convencional - COPSE/CUTE");
		paper.setLevel (BW_Paper.MAXIMUM);
		
		BW_Profile paper_profile = UML_BW_Mediator.makeProfile();
		paper_profile.setTypeId (typeIdDefault.BW_PAPERPROFILE);
		paper.setProfile (paper_profile);
		
		return paper;
	}

	/** cria o grupo default */
	public static BW_Group makeDefaultGroup () {
		BW_Group group = new BW_Group (UML_BW_Mediator.GROUPTYPEID,UML_BW_Mediator.GROUPOBJID);
		group.setName("COPSE/CUTE");
		group.setDescription ("Grupo de usuarios do COPSE/CUTE");
		return group;
	}
	
	/** criando registro de eventos default */
	public static BW_Register makeDefaultRegister () {
	        BW_Register register = new BW_Register();
	        Vector prototipos =  UML_BW_Mediator.getPrototypes();
	        for (int i=0; i < prototipos.size(); i++) {
	        	register.insertEvent ((BW_Event)prototipos.elementAt(i));
	        }
	        return register;
	}
	

	/** criando ST_Setup default */
	public static ST_Setup makeDefaultSetup (String host) {
		ST_Setup setup = new ST_Setup();
		setup.setServer (host);
		setup.setLastObjId (UML_BW_Mediator.lastObjId);
		if (host.equals("localhost"))
			setup.setIsServer (true);
		else
			setup.setIsServer (false);
		
		return setup;
	}
	
	
	/////////////////////////////////////////////
	public static void main (String argv[]) {
		String host="localhost";
		
	    	TimeZone tmz = TimeZone.getTimeZone ("BET");
	    	Locale lcl = new Locale ("pt","BR");
	    	TimeZone.setDefault (tmz);
	    	Locale.setDefault (lcl);
	
                ST_DatabaseImplementor db = new ST_DatabaseImplementor (host);
				
		if (argv.length>=2) {
                    // argumentos: -h host
                    for (int i=0; i<argv.length;i++){
                    	String arg = argv[i];
                    	if (arg.charAt(0)=='-') {
                    		switch (arg.charAt(1)) {
                    			case 'h':
                    				host= new String (argv[i+1]);
                    		                break;	
                    			default:
                    				System.out.println ("Use: java BW.UML.UML_Setup [-h host]");
                    				System.exit(0);
                    		}
                    	}
                    }
                }


                // criando o grupo default
                BW_Group groupD = makeDefaultGroup();

                //criando papel default
                BW_Paper paperD = makeDefaultPaper();

                //criando usuario default
                BW_Member memberD = makeDefaultMember (paperD);

                groupD.insertPaper(paperD);
                groupD.insertMember(memberD);

                //criando registro default
                BW_Register registerD = makeDefaultRegister();

                //criando setup default
                ST_Setup setupD = makeDefaultSetup(host);

                //gravando tudo
                System.out.println ("Inserting...");  		
                System.out.println ("Group: "+groupD);
                db.write (groupD);
                System.out.println ("Register: "+registerD);
                db.write (registerD);
                System.out.println ("Setup: "+setupD);
                db.write (setupD);
                System.out.println ("Closing...");
                db.close();
	}
}
