/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CLIENT
 * Classe:  CGUICollaborationManager
\******************************************************************************/

package COPSE.CLIENT.GUI;


import  java.awt.*;
//import  java.awt.event.*;

import  COPSE.*;
import  COPSE.CLIENT.*;
import  COPSE.CONTROL.*;

/* **************************************** BW ********************************************* */
import  BW.*;
import  BW.Interface.*;
/* **************************************** BW ********************************************* */

public class CGUICollaborationManager
    extends     CEventHandler
    implements  CClientSessionManagerListener, CClientFileControllerListener//, CClientCollaborationListener
{
    protected   CGUIManager                 pGUI;
    protected   CClient                     pClient;


    // Specific Tool Manager...
    //protected   CGUIToolManager             pTool;
    protected   boolean                     mIsClientOpen;

    //* SessionManager and FileController
    protected   CClientSessionManager       pSessionManager;
    protected   CFileController             pFileController;


    //* Menus
    protected   Menu                        pAwarenessMenu;
    protected   Menu                        pCommunicationMenu;
    protected   Menu                        pCoordinationMenu;
    protected   Menu                        pGroupMemoryMenu;

    //* GUI Controls...
    // Awareness...
    protected   CGUIMultiPointerControl     pMultiPointerControl;   // "many telepointers"
    protected   CGUIGestaltControl          pGestaltControl;        // Miniview
    protected   CGUIUserListControl         pUserListControl;       // or "snapshot"
    protected   CGUIMultiScrollbarControl   pMultiScrollbarControl; // scrollbar with

/* **************************************** BW ********************************************* */
    protected	UI_GUIElement               pPastAwarenessControl;  // past events awareness
/* **************************************** BW ********************************************* */

    // Communication
    protected   CGUIChatControl             pChatControl;
    protected   CGUISketchpadControl        pSketchpadControl;
    protected   CGUIMessageControl          pMessageControl;

    // Coordination
    protected   CGUILockObjectControl       pLockObjectControl;     // ??

    // GroupMemory
    protected   CGUIAnnotationControl       pAnnotationControl;
    protected   CGUIHistoryControl          pHistoryControl;
    protected   CGUILinksControl            pLinksControl;
    protected   CGUISuggestionControl       pSuggestionControl;


    public  CGUICollaborationManager ( CGUIManager gui )
    {
        pGUI = gui;
        pClient = null;
        mIsClientOpen = false;
        CreateControls();
    }

    public CGUIManager  GetGUIManager()
    {   return pGUI;  }


    protected void ChangeMenuSettings()
    {
        pAwarenessMenu.setEnabled( mIsClientOpen );
        pCommunicationMenu.setEnabled( mIsClientOpen );
        pCoordinationMenu.setEnabled( mIsClientOpen );
        pGroupMemoryMenu.setEnabled( mIsClientOpen );
    }

    protected void CreateControls()
    {
        pAwarenessMenu      = new Menu( "Awareness" );
        pCommunicationMenu  = new Menu( "Communication" );
        pCoordinationMenu   = new Menu( "Coordination" );
        pGroupMemoryMenu    = new Menu( "GroupMemory" );

        CreateAwarenessControls( pAwarenessMenu );
        CreateCommunicationControls( pCommunicationMenu );
        CreateCoordinationControls( pCoordinationMenu );
        CreateGroupMemoryControls( pGroupMemoryMenu );

        ChangeMenuSettings();
    }


    protected void CreateAwarenessControls( Menu menu )
    {
        pMultiPointerControl    = new CGUIMultiPointerControl( this, menu );
        pGestaltControl         = new CGUIGestaltControl( this, menu );
        pUserListControl        = new CGUIUserListControl( this, menu );
        pMultiScrollbarControl  = new CGUIMultiScrollbarControl( this, menu );

/* **************************************** BW ********************************************* */
	pPastAwarenessControl	= GW_BW_Mediator.GetMainUIElement();
	if (pPastAwarenessControl != null) {
		CheckboxMenuItem pastAwarenessItem  = new CheckboxMenuItem ("Past Events");
		pastAwarenessItem.addItemListener (pPastAwarenessControl);
		pastAwarenessItem.setState (false);
		menu.add (pastAwarenessItem);
		pPastAwarenessControl.setMenuItem (pastAwarenessItem);
		
		System.out.println ("[DBG]COPSE.CLIENT.GUI.CGUICollaborationManager.CreateAwarenessControls: "+
					" Item for Past Events Awareness Added");
	}
	else {
		System.out.println ("[DBG]COPSE.CLIENT.GUI.CGUICollaborationManager.CreateAwarenessControls: "+
					" Impossible to add Past Events Awareness item");	
	}
/* **************************************** BW ********************************************* */

    }


    protected void CreateCommunicationControls( Menu menu )
    {
        pChatControl            = new CGUIChatControl( this, menu );
        pSketchpadControl       = new CGUISketchpadControl( this, menu );
        pMessageControl         = new CGUIMessageControl( this, menu );
    }


    protected void CreateCoordinationControls( Menu menu )
    {
        pLockObjectControl      = new CGUILockObjectControl( this, menu );
    }

    protected void CreateGroupMemoryControls( Menu menu )
    {
        pAnnotationControl  = new CGUIAnnotationControl( this, menu );
        pHistoryControl     = new CGUIHistoryControl( this, menu );
        pLinksControl       = new CGUILinksControl( this, menu );
        pSuggestionControl  = new CGUISuggestionControl( this, menu );
    }


    protected void TurnOffAllControls()
    {
        pMultiPointerControl.SetEnabled( false );
        pGestaltControl.SetEnabled( false );
        pUserListControl.SetEnabled( false );
        pMultiScrollbarControl.SetEnabled( false );

        pChatControl.SetEnabled( false );
        pSketchpadControl.SetEnabled( false );
        pMessageControl.SetEnabled( false );

        pLockObjectControl.SetEnabled( false );

        pAnnotationControl.SetEnabled( false );
        pHistoryControl.SetEnabled( false );
        pLinksControl.SetEnabled( false );
        pSuggestionControl.SetEnabled( false );
    }


    public void AddMenu( MenuBar menubar )
    {
        menubar.add( pAwarenessMenu );
        menubar.add( pCommunicationMenu );
        menubar.add( pCoordinationMenu );
        menubar.add( pGroupMemoryMenu );
    }


    public void SetClient( CClient client )
    {
        pClient = client;

        mIsClientOpen = pClient != null;

        ChangeMenuSettings();


        if( !mIsClientOpen )
        {
            TurnOffAllControls();
            return;
        }


        // Start Listening Client ...
        pClient.AddSessionManagerListener( this );
        pClient.AddFileControllerListener( this );
        pClient.GetCollaborationManager().SetListener( this );

    }







    protected void ProcessEvent( CEvent event )
    {
        int type = event.GetType();

        if( (type < CEvent.EvCollaboration_First) ||
            (type > CEvent.EvCollaboration_Last)  )
        return;


        // Awareness...
        if( (type >= CEvent.EvCollaboration_Awareness_Pointer_First) &&
            (type <= CEvent.EvCollaboration_Awareness_Pointer_Last)  )
            pMultiPointerControl.AddEvent( event );
        else

        if( (type >= CEvent.EvCollaboration_Awareness_Screen_First) &&
            (type <= CEvent.EvCollaboration_Awareness_Screen_Last)  )
        {
            pGestaltControl.AddEvent( event );
            pMultiScrollbarControl.AddEvent( event );
        }
        else

        // Communication ...
        if( type == CEvent.EvCollaboration_Communication_Call_Message )
            pMessageControl.AddEvent( event );
        else

        if( type == CEvent.EvCollaboration_Communication_Chat_Message )
            pChatControl.AddEvent( event );
        else

        {

        }

    }


    //
    protected void ActivateFileControls()
    {

    }

    protected void UpdateFileControls()
    {

    }


    // CClientFileControllerListener
    public CFileController      GetFileController()     { return pFileController; }

    public void  FileControllerLoaded()
    {
        System.err.println( "CGUICollaborationManager - FileController was loaded!" );
        pFileController = pClient.GetFileController();
        ActivateFileControls();
    }

    public void  ElementStateChanged (int idElem, CEvent event)
    {
        UpdateFileControls();
    }






    //
    protected void ActivateSessionControls()
    {
        pChatControl.Activate();
        pMessageControl.Activate();

        pUserListControl.Activate();
    }

    protected void UpdateSessionControls()
    {
        pChatControl.Update();
        pMessageControl.Update();

        pUserListControl.Update();
    }



    // CClientSessionManager Listener
    public CClientSessionManager      GetSessionManager()     { return pSessionManager; }

    public void  SessionManagerLoaded()
    {
        System.err.println( "CGUICollaborationManager - SessionManager was loaded!" );
        pSessionManager = pClient.GetSessionManager();
        pSessionManager.DEBUG_write( System.err );
        ActivateSessionControls();
    }

    public void  ConnectionOpened (int connId)
    {
        UpdateSessionControls();
        System.err.println( "CGUICollaborationManager - New Connection ["+connId+"] !" );
    }

    public void  ConnectionClosed (int connId)
    {
        UpdateSessionControls();
        System.err.println( "CGUICollaborationManager - Connection closed ["+connId+"] !" );
    }



    public void Exit()
    {
        ExitAllControls();
        this.stop();
    }

    protected void ExitAllControls()
    {
        pMultiPointerControl.Exit();
        pGestaltControl.Exit();
        pUserListControl.Exit();
        pMultiScrollbarControl.Exit();

        pChatControl.Exit();
        pSketchpadControl.Exit();
        pMessageControl.Exit();

        pLockObjectControl.Exit();

        pAnnotationControl.Exit();
        pHistoryControl.Exit();
        pLinksControl.Exit();
        pSuggestionControl.Exit();
    }


}