/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CONTROL
 * Classe:  CFileController
\******************************************************************************/

package COPSE.CONTROL;

import java.io.*;
import java.util.*;

import FILE.*;


public class CFileController
    implements Serializable
{
    protected   CFile       pFile;

    protected   Vector      pElementCtrls;

    public CFileController( CFile file )
    {
        pFile = file;
        InitController();
    }

    private void  InitController()
    {
        pElementCtrls = new Vector();

        Enumeration e = pFile.GetElements();
        while( e.hasMoreElements() )
        {
            pElementCtrls.addElement( CreateElementController( (CElement)e.nextElement() ) );
        }
    }

    protected CElementController    CreateElementController( CElement elem )
    {   return new CElementController( elem );  }


    public  Enumeration     GetElementControllers() { return pElementCtrls.elements(); }

    public  CFile           GetFile()               { return pFile; }


    public CElementController   FindElementController( int elemId )
    {
        CElementController controller;
        Enumeration e = pElementCtrls.elements();
        while( e.hasMoreElements() )
        {
            controller = (CElementController) e.nextElement();
            if( controller.GetElement().GetId() == elemId )
                return controller;
        }

        return null;
    }

    public synchronized boolean  LockElement( int connId, CElementController control )
    {
        return control.Lock( connId );
    }


    public synchronized boolean  LockElement( int connId, int elemId )
    {
        CElementController elem = FindElementController( elemId );
        if( elem == null )
            return false;
        return LockElement( connId, elem );
    }


    public synchronized boolean  UnlockElement( int connId, CElementController control )
    {
        return control.Unlock( connId );
    }


    public synchronized boolean  UnlockElement( int connId, int elemId )
    {
        CElementController elem = FindElementController( elemId );
        if( elem == null )
            return false;
        return UnlockElement( connId, elem );
    }


    public synchronized void     NewElement( int connId, CElement elem )
    {
        pFile.AddElement( elem );
        pElementCtrls.addElement( CreateElementController(elem) );

        //TODO: History - to record the creation (who,when...) of this element
    }

    public synchronized boolean  DeleteElement( int connId, int elemId )
    {
        CElementController control = FindElementController( elemId );
        if( control == null )   return false;

        if( CanRemoveElement( connId, control ) )
        {
            pFile.RemoveElement( control.GetElement() );
            pElementCtrls.removeElement( control );
            return true;
        }
        return false;
    }


    public synchronized boolean  UpdateElement( int connId, CElement elemUpdate )
    {
        CElementController control = FindElementController( elemUpdate.GetId() );
        if( control == null ) return false;

        if( control.GetLockerConnectionId() != connId )
            return false;

        control.GetElement().UpdateFrom( elemUpdate );
        return true;
    }

    public synchronized boolean  CanRemoveElement ( int connId, CElementController control )
    {
        if( control.IsLocked() && (control.GetLockerConnectionId() != connId) )
            return false;
        return true;
    }


    public synchronized boolean  CanRemoveElement ( int connId, int elemId )
    {
        CElementController elem = FindElementController( elemId );
        if( elem == null )
            return false;
        return CanRemoveElement( connId, elem );
    }


}
