/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE.SERVER
 * Classe: CServer
\******************************************************************************/

package COPSE.SERVER;

import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import FILE.*;




public class CQueuedServerManager
    extends CServerManager
{
    protected Vector pEvents;

    public CQueuedServerManager ( int port, CLocator locator, CFile file )
        throws Exception
    {
        super( port, locator, file );

        pEvents = new Vector();
    }

    protected void CreateServerSocket( int port )
    {
        mServerSocketPort = port;
        mServerSocketHost = "localhost";
    }



    public void AddEvent( CEvent event, CConnectionHandler globalUserConnection )
    {
        synchronized( pEvents )
        {
            pEvents.addElement( event );
            pEvents.addElement( globalUserConnection );
            pEvents.notifyAll();
        }
    }




    public void run()
    {
        System.err.println( "[DEBUG] CQueuedServerManager: running... (accepting new connections)" );
        System.err.flush();

        // Accepting new connections from User Clients
        while( !mExit )
        {
            CEvent event;
            CConnectionHandler handler;

            System.err.println( "[DEBUG] CQueuedServerManager - waiting for new connection request..." );

            synchronized( pEvents )
            {
                while( pEvents.isEmpty() )
                {
                    try {
                        pEvents.wait(100);
                    } catch (InterruptedException ie)
                    {
                        if( mExit ) break;
                    }
                }

                if( mExit ) break;

                if( pEvents.size() < 2 )
                {
                    pEvents.removeAllElements();
                    continue;
                }

                System.err.println( "[DEBUG] CQueuedServerManager - receiving an event.." );
                try {
                    event = (CEvent) pEvents.elementAt(0);
                    handler = (CConnectionHandler) pEvents.elementAt(1);
                } catch ( Exception e )
                {
                    e.printStackTrace( System.err );
                    pEvents.removeElementAt(1);
                    pEvents.removeElementAt(0);
                    continue;
                }

                pEvents.removeElementAt(1);
                pEvents.removeElementAt(0);
            }

            System.err.println( "[DEBUG] CQueuedServerManager - connecting user..." );
            _ConnectQueuedUser( event, handler );

        } // end while( !mExit )
    }





    private CUserConnectionHandler CreateUserQueuedConnectionHandler( int userId, int toolId,
        CConnectionHandler globalUserHandler )
    {
        int connId = pSessionManager.CreateNewConnectionId();

        CServerUserSession userSession = new CServerUserSession( userId, connId );

        // added to the original
//        CEventQueueConnector connector = new CEventQueueConnector( GetServerSocketPort(), connId, globalUserHandler.GetConnector() );
        // Initially, the connectionId has to be 0 so as to the client be able to receive the open_connection event answer
        // Just after sending the open_connection event answer, this connector will receive the correct connectionId
        CEventQueueConnector connector = new CEventQueueConnector( GetServerSocketPort(), 0, globalUserHandler.GetConnector() );

        CUserConnectionHandler pConnectionHandler = new CUserConnectionHandler( connector, pEventHandler, userSession );

        CServerToolSession toolSession = (CServerToolSession) pSessionManager.FindToolSession( toolId );
        if( toolSession == null )
        {
            toolSession = new CServerToolSession( toolId );
            pSessionManager.AddToolSession( toolSession );
        }

        toolSession.AddUserSession( userSession );

        pConnectionHandler.start();
        return pConnectionHandler;
    }



    private void _ConnectQueuedUser( CEvent event, CConnectionHandler globalUserHandler )
    {

        if( event.GetType() == CEvent.EvCOPSE_Connection_Open )
        {
            // Get User Identification
            Integer userId = (Integer) event.GetArgumentAt(0);
            Integer toolId = (Integer) event.GetArgumentAt(1);

            System.err.println( "[DEBUG] CQueuedServerManager - User "+userId+" on tool("+toolId+") conecting..." );

            // Create the Conection Handler:
            // This includes the user to the SessionManager
            CUserConnectionHandler pConnection = CreateUserQueuedConnectionHandler(
                userId.intValue(), toolId.intValue(), globalUserHandler );


            System.err.println( "[DEBUG] CQueuedServerManager - Notifying everyone on this server..." );

            event = new CEvent( CEvent.EvCOPSE_Connection_Open );
            event.AddArgument( new Integer(pConnection.GetConnectionId()) );
            event.AddArgument( userId );
            event.AddArgument( toolId );

            // Notify all users about this new connection
            pSessionManager.NotifyAll( event );

            // After sending the open_connection answer event, it sets the connector to the right socketPort value
            ((CEventQueueConnector)pConnection.GetConnector()).SetConnectionId( pConnection.GetConnectionId() );

            // that's all
            System.err.println( "[DEBUG] CQueuedServerManager - New user is connected!" );
            return;
        }
        else
        {
            try {
                event = new CEvent( CEvent.EvCOPSE_Connection_Error );
                event.AddArgument( new String("Invalid command. Open Connection command expected!") );
                globalUserHandler.Send( event );
            } catch( Exception e ) {};
        }
    }


    public CEventQueueConnector FindQueueConnector( int connectionId )
    {
        //This function should be improved to spend less time
        CServerUserSession userSession;
        CConnectionHandler handler;

        userSession = (CServerUserSession) pSessionManager.FindSessionByConnection( connectionId );
        if( userSession != null )
        {
            handler = userSession.GetConnectionHandler();
            return (CEventQueueConnector)(handler.GetConnector());
        }
        return null;
    }

/*
    public void Exit()
    {
        super.Exit();

        synchronized( pEvents )
        {
            pEvents.notifyAll();
        }
    }
*/

}

