/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE
 * Classe:  CVersion
\******************************************************************************/


package FILE;



import java.io.*;
import java.util.*;


public class    CVersion
    extends     CElement
{
    protected   int             mParentId;
    protected   String          mDescription;

    protected   CDocument       pDocument;

    // The file is not to be serialized with version
    protected transient CFile   tFile;


    //** Constructor
    public CVersion( CDocument doc )
    {   super();
        mDescription = "";
        mParentId=0;

        pDocument = doc;
        tFile = null;
    }

    //** Attributes
    public void     SetDescription( String desc )   { mDescription = desc; }
    public String   GetDescription( )               { return mDescription; }


    public void     SetParentId( int id )           { mParentId = id; }
    public int      GetParentId()                   { return mParentId; }

    public void         SetDocument( CDocument doc ){ pDocument = doc; }
    public CDocument    GetDocument( )              { return pDocument; }


    public CFile    LoadFile( String productDir )
        throws IOException, ClassNotFoundException
    {
        String  name = GetFileName();
        tFile = CFile.Load( name, new String( productDir + "/" + pDocument.GetPath() ) );
        return tFile;
    }

    public String   GetFileName()
    {
        return new String( pDocument.GetName() + ".v" + String.valueOf( mId ) );
    }

    public CFile    GetFile( String productDir )
    {
        if( tFile != null )
            return tFile;
        else
        try
        {
            return LoadFile( productDir );
        }
        catch( Exception e )
        {
            return null;
        }
    }

    public void     SetFile ( CFile file )
    {
        tFile = file;

        String name = pDocument.GetName() + ".v" + String.valueOf( mId );
        tFile.SetName( name );
        tFile.SetPath( pDocument.GetPath() );
    }



    /* * Serializable
    protected void writeObject(java.io.ObjectOutputStream out)
        throws IOException
    {
        System.out.println( "DEBUG: CVersion.writeObject" );
        super.writeObject( out );
        out.writeInt( mParentId );
        out.writeObject( mDescription );
    }

    protected void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException
    {
        System.out.println( "DEBUG: CVersion.readObject" );
        super.readObject( in );
        mParentId = in.readInt();
        mDescription = (String) in.readObject();
    }
/*   */

    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CVersion");
        pStr.println( "       Parent Id  :     " + mParentId );
        pStr.println( "       Description:     " + mDescription );
        super.DEBUG_write( pStr );
    }


}
