/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE.OBJ
 * Classe:  CObjRelationExtremity
\******************************************************************************/

package FILE.OBJ;

import  java.io.*;
import  java.awt.Rectangle;
import  java.awt.Point;

import  GRAPH.CSidePosition;


/*************************************************************************
 *  Class CObjRelationExtremity
 *************************************************************************/
public class    CObjRelationExtremity
    implements  Serializable
{
    protected   CObjRelation    pRelation;
    protected   CObjClass       pClass;


    public static final int     ExtTop          = 0;
    public static final int     ExtBottom       = 1;
    public static final int     ExtLeft         = 2;
    public static final int     ExtRight        = 3;

    protected   CSidePosition   pSidePosition;
/*
    protected   int             mClassSide;
    protected   int             mSideOffset;
*/

    protected   boolean         mArrow; // with or without arrow ? (Navegability)

    protected   CObjCardinality pCardinality;
    protected   CObjLabel       pLabel;


    public CObjRelationExtremity( CObjRelation relation, CObjClass aClass )
    {
        this( relation );

        pClass = aClass;
        pClass.AddRelation( pRelation );
    }


    public CObjRelationExtremity( CObjRelation relation )
    {
        pRelation = relation;
        pRelation.AddExtremity( this );

        pClass = null;

//        mClassSide = ExtRight;
//        mSideOffset = 0;
        pSidePosition = new CSidePosition( CSidePosition.RIGHT, 0 );

        mArrow = false;

        pCardinality = new CObjCardinality("");
        pLabel = new CObjLabel("");
    }



    public CObjCardinality  GetCardinality ()               { return pCardinality; }
    public CObjLabel        GetLabel ()                     { return pLabel; }
    public boolean          HasArrow ()                     { return mArrow; }

    public CObjClass        GetClass ()                     { return pClass; }

    public int              GetClassSide ()                 { return pSidePosition.GetSide(); }

    public CSidePosition    GetSidePosition ()              { return pSidePosition; }


    public void     SetCardinality (CObjCardinality card)   { pCardinality = card; }
    public void     SetLabel ( CObjLabel label )            { pLabel = label; }
    public void     SetArrow ( boolean arrow )              { mArrow = arrow; }


    public void     SetClass (CObjClass aClass)
    {
        if(pClass != null)
            pClass.RemoveRelation( pRelation );

        pClass = aClass;

        if(pClass != null)
            pClass.AddRelation( pRelation );
    }




    public void     AdjustPoint     ( Point pIn, Point pOut )
    {
        Rectangle rect = pClass.GetRegion();

        CSidePosition sidePos = CSidePosition.CreateSidePosition( rect, pIn, pOut );

        SetPoint( sidePos.GetSide(), sidePos.GetPosition() );
    }

/*
        if( pIn.x == pOut.x )
        {
            mSideOffset = pIn.x - rect.x;

            if( pOut.y < rect.y )  // pOut is above
            {
                mClassSide = 0;
                //pIn.y = rect.y;
            }
            else                   // pOut is below
            {
                mClassSide = 1;
                //pIn.y = rect.y + rect.height;
            }

            return;
        }
        else
        if( pIn.y == pOut.y )
        {
            mSideOffset = pIn.y - rect.y;

            if( pOut.x < rect.x )  // pOut at left
            {
                mClassSide = 2;
                //pIn.x = rect.x;
            }
            else                   // pOut at right
            {
                mClassSide = 3;
                //pIn.x = rect.x + rect.width;
            }

            return;
        }
        else
        {
            float a, b;
            int x, y;

            a = ((float)          ( pIn.y - pOut.y )          ) / (pIn.x - pOut.x);
            b = ((float) ( pIn.x * pOut.y - pOut.x * pIn.y )  ) / (pIn.x - pOut.x);

            Rectangle r = new Rectangle( pIn.x, pIn.y, pOut.x - pIn.x, pOut.y - pIn.y );


            Point p0, p1, p2, p3;
            y = rect.y;                     p0 = new Point( (int)((y - b) / a), y );
            y = rect.y + rect.height;       p1 = new Point( (int)((y - b) / a), y );
            x = rect.x;                     p2 = new Point( x, (int)(a*x + b) );
            x = rect.x + rect.width;        p3 = new Point( x, (int)(a*x + b) );


            if( r.contains(p0) )
            {
                int dist0 = (pIn.x-p0.x)*(pIn.x-p0.x) + (pIn.y-p0.y)*(pIn.y-p0.y);
                int dist;

                if( r.contains(p2) )
                {
                    dist = (pIn.x-p2.x)*(pIn.x-p2.x) + (pIn.y-p2.y)*(pIn.y-p2.y);
                    if( dist0 < dist )
                        SetPoint( ExtTop, p0.x - rect.x );
                    else
                        SetPoint( ExtLeft, p2.y - rect.y );
                }
                else
                if( r.contains(p3) )
                {
                    dist = (pIn.x-p3.x)*(pIn.x-p3.x) + (pIn.y-p3.y)*(pIn.y-p3.y);
                    if( dist0 < dist )
                        SetPoint( ExtTop, p0.x - rect.x );
                    else
                        SetPoint( ExtRight, p3.y - rect.y );
                }
                else
                    SetPoint( ExtTop, p0.x - rect.x );
            }
            else

            if( r.contains(p1) )
            {
                int dist1 = (pIn.x-p1.x)*(pIn.x-p1.x) + (pIn.y-p1.y)*(pIn.y-p1.y);
                int dist;

                if( r.contains(p2) )
                {
                    dist = (pIn.x-p2.x)*(pIn.x-p2.x) + (pIn.y-p2.y)*(pIn.y-p2.y);
                    if( dist1 < dist )
                        SetPoint( ExtBottom, p1.x - rect.x );
                    else
                        SetPoint( ExtLeft, p2.y - rect.y );
                }
                else
                if( r.contains(p3) )
                {
                    dist = (pIn.x-p3.x)*(pIn.x-p3.x) + (pIn.y-p3.y)*(pIn.y-p3.y);
                    if( dist1 < dist )
                        SetPoint( ExtBottom, p1.x - rect.x );
                    else
                        SetPoint( ExtRight, p3.y - rect.y );
                }
                else
                    SetPoint( ExtBottom, p1.x - rect.x );
            }
            else

            if( r.contains(p2) )
                SetPoint( ExtLeft, p2.y - rect.y );

            else
                SetPoint( ExtRight, p3.y - rect.y );
        }

    } // AdjustPoint...
*/

    public void SetSidePosition( CSidePosition sidePosition )
    {
        pSidePosition.Set( sidePosition.GetSide(), sidePosition.GetPosition() );
    }

    public void SetPoint( int side, int position )
    {
        //mClassSide = side;
        //mSideOffset = offset;
        pSidePosition.SetSide( side );
        pSidePosition.SetPosition( position );
    }


    public Point GetPoint()
    {
        Rectangle rect = pClass.GetRegion();
        return pSidePosition.GetPoint( rect );
        /*
        int x, y;

        switch( pSidePosition.GetSide() )
        {
            case ExtTop:
                x = rect.x + pSidePosition.GetPosition();
                y = rect.y;
                break;
            case ExtBottom:
                x = rect.x + pSidePosition.GetPosition();
                y = rect.y + rect.height;
                break;
            case ExtLeft:
                x = rect.x;
                y = rect.y + pSidePosition.GetPosition();
                break;
            case ExtRight:
            default:
                x = rect.x + rect.width;
                y = rect.y + pSidePosition.GetPosition();
        }

        return new Point(x,y);
        /* */
    }


    public void CopyInfoFrom( CObjRelationExtremity extr )
    {
        pSidePosition.SetSide( extr.pSidePosition.GetSide() );
        pSidePosition.SetPosition( extr.pSidePosition.GetPosition() );
//        mClassSide =    extr.mClassSide;
//        mSideOffset =   extr.mSideOffset;
        mArrow =        extr.mArrow;

        pCardinality =  extr.GetCardinality();
        pLabel =        extr.GetLabel();
    }


    public void CopyFrom( CObjRelationExtremity extr )
    {
        pClass =        extr.GetClass();

        pSidePosition.SetSide( extr.pSidePosition.GetSide() );
        pSidePosition.SetPosition( extr.pSidePosition.GetPosition() );
//        mClassSide =    extr.mClassSide;
//        mSideOffset =   extr.mSideOffset;
        mArrow =        extr.mArrow;

        pCardinality =  (CObjCardinality)   extr.GetCardinality().Clone();
        pLabel =        (CObjLabel)         extr.GetLabel().Clone();
    }


    public void DEBUG_write( PrintStream pStr )
    {
        pStr.print( "RelExtremity:  Class [" );
        if( pClass != null )
            pStr.print( pClass.GetName() );
        pStr.println( "[ Card[" + pCardinality.GetText() + "] Label[" + pLabel.GetText() + "]" );
    }

}



/*
            // side 0: Top
            y = rect.y;
            x = (int)((y - b) / a);
            if( r.inside( x, y ) )
            {
                mClassSide = ExtTop;
                mSideOffset = x - rect.x;
                pIn.move( x, y );
                return;
            }

            // side 1: Bottom
            y = rect.y + rect.height;
            x = (int)((y - b) / a);
            if( r.inside( x, y ) )
            {
                mClassSide = ExtBottom;
                mSideOffset = x - rect.x;
                pIn.move( x, y );
                return;
            }

            // side 2: Left
            x = rect.x;
            y = (int)(a*x + b);
            if( r.inside(x,y) )
            {
                mClassSide = ExtLeft;
                mSideOffset = y - rect.y;
                pIn.move( x, y );
                return;
            }

            // side 3: Right
            x = rect.x + rect.width;
            y = (int)(a*x + b);
            if( r.inside(x,y) )
            {
                mClassSide = ExtRight;
                mSideOffset = y - rect.y;
                pIn.move( x, y );
                return;
            }
        }
    }


*/






