/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CSelectorPoint
\******************************************************************************/

package GRAPH;

import java.awt.*;


public class CSelectorPoint
{
    // This class works with REAL coordinates, and doesn't with SCALED coordinates !!

    protected   Point       pPoint;
    protected   Rectangle   pRect;

    // SelectorPoint Type
    public static final int DEFAULT     = 0; // ARROW cursor
    public static final int MOVE        = 1;
    public static final int N_RESIZE    = 2;
    public static final int S_RESIZE    = 3;
    public static final int E_RESIZE    = 4;
    public static final int W_RESIZE    = 5;
    public static final int NE_RESIZE   = 6;
    public static final int SE_RESIZE   = 7;
    public static final int NW_RESIZE   = 8;
    public static final int SW_RESIZE   = 9;

    protected   int         mType;
    protected   Cursor      pCursor;



    public CSelectorPoint( Point p )
    {
        this( p, MOVE );
    }

    public CSelectorPoint( Point p, int type )
    {
        SetType( type );
        SetPoint( p );
    }

    public void  SetPoint( Point p )
    {
        pPoint = p;
        pRect = new Rectangle( p.x-3, p.y-3, 6, 6 );
    }

    public void  SetType( int type )
    {
        if( (type<0) || (type>9) )  return;

        mType = type;

        int table [] =
        {   Cursor.DEFAULT_CURSOR,     Cursor.MOVE_CURSOR,        Cursor.N_RESIZE_CURSOR,    Cursor.S_RESIZE_CURSOR,
            Cursor.E_RESIZE_CURSOR,    Cursor.W_RESIZE_CURSOR,    Cursor.NE_RESIZE_CURSOR,   Cursor.SE_RESIZE_CURSOR,
            Cursor.NW_RESIZE_CURSOR,   Cursor.SW_RESIZE_CURSOR
        };

        pCursor = Cursor.getPredefinedCursor( table[mType] );
    }


    public Point    GetPoint()      { return pPoint; }

    public int      GetType()       { return mType; }

    public Cursor   GetCursor()     { return pCursor; }

    public boolean  Contains ( int x, int y )
    {   return pRect.contains(x,y); }


    public void Draw( CGraphics g )
    {
        //Graphics g = cg.GetGraphics();

        g.setXORMode( Color.white );
        g.RealFillRect( pRect.x, pRect.y, pRect.width, pRect.height );
        g.setPaintMode();
    }

/*
    public void     MouseMove   ( CGraphics g, int x, int y )
    {
        // Hide itself
        g.RealFillRect( pRect.x, pRect.y, pRect.width, pRect.height );

        // Set new point
        SetPoint( new Point(x,y) );

        // Show itself
        g.RealFillRect( pRect.x, pRect.y, pRect.width, pRect.height );
    }
*/

}