
/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: PROCESS
 * Classe:  CAbstractActivity
\******************************************************************************/

package PROCESS2;

import java.io.*;
import java.util.*;

import FILE.CElement;
import FILE.CFile;


public abstract class CAbstractActivity
    extends CElement
{
    protected   String              mName;
    protected   String              mDescription;
    protected   int                 mToolId;

//    protected   Rectangle           mRegion;

    protected   CAbstractActivity   pParent;    // ??

    protected   Vector              pSubActivities;
    protected   Vector              pRoles;
    protected   Vector              pFlows;   // ??


    public  CAbstractActivity()
    {
        super();

        mName = "";
        mToolId = 0;
        mDescription = "";

        pParent = null;

        pSubActivities = new Vector();

        pRoles = new Vector();
        pFlows = new Vector();
    }



    // Name
    public String   GetName()               { return mName; }
    public void     SetName(String name)    { mName = name; }


    // Description
    public String   GetDescription()            { return mDescription; }
    public void     SetDescription(String desc) { mDescription = desc; }



    //** Parent Activity *********

    /**
     * Get the parent Activity.
     * 'null' means that this activity doesn't have a parent activity - it's
     * the whole process
     */
    public CAbstractActivity     GetParent ()           { return pParent; }
    protected void SetParent (CAbstractActivity parent) { pParent = parent; }



    /**
     * Add a sub-activity to this activity.
     */
    public void AddActivity( CAbstractActivity act )
    {
        AddActivity( act, true );
    }

    public void AddActivity( CAbstractActivity act, boolean addToFile )
    {
        if( addToFile )
        {
            CFile file = GetFile();
            if( file != null )
                file.AddElement( act );
        }

        pSubActivities.addElement( act );
        act.SetParent( this );
    }

    public boolean RemoveActivity( CAbstractActivity act )
    {
        CFile file = GetFile();
        if( file != null )
            file.RemoveElement( act );
        return pSubActivities.removeElement( act );

        //To DO: Remove subactivities/flows of act from file...
    }


    public boolean      HasActivities()
    {
        return( pSubActivities.size() != 0 );
    }

    public Enumeration  GetActivities ()
    {
        return pSubActivities.elements();
    }





    //** Tool *****************

    /**
     * Verify if this activity is a final activity - an activity defined by a tool.
     */
    public boolean  IsDefinedByTool ()              { return mToolId != 0; }

    /**
     * Get the Tool Id - 0 means it is not a final activity
     */
    public int      GetToolId ()                    { return mToolId; }
    public void     SetToolId ( int id )            { mToolId = id; }






    //** Roles *****************
    public void         AddRole    ( CAbstractRole role )   { pRoles.addElement( role ); }
    public boolean      RemoveRole ( CAbstractRole role )   { return pRoles.removeElement( role ); }
    public Enumeration  GetRoles   ()                       { return pRoles.elements(); }



    //** Flows *****************
    public boolean      AddFlow    ( CFlow flow )
    {
        if( flow.GetParent() != this )
            return false;

        CFile file = GetFile();
        if( file != null )
            file.AddElement( flow );
        pFlows.addElement( flow );
        return true;
    }

    public boolean      RemoveFlow ( CFlow flow )
    {
        CFile file = GetFile();
        if( file != null )
            file.RemoveElement( flow );
        return pFlows.removeElement( flow );
    }

    public Enumeration  GetFlows   ()
    { return pFlows.elements(); }




    protected Enumeration GetFromActivities( CAbstractActivity act )
    {
        Vector      pFrom = new Vector();

        if( act.GetParent() != this )
            return pFrom.elements();

        Enumeration flows = GetFlows();
        while( flows.hasMoreElements() )
        {
            CFlow flow = (CFlow) flows.nextElement();

            Enumeration e = flow.GetToActivities();
            while( e.hasMoreElements() )
            {
                if( act == e.nextElement() )
                {
                    Enumeration from = flow.GetFromActivities();
                    while( from.hasMoreElements() )
                        pFrom.addElement( from.nextElement() );

                    break; // break -> while( e.hasMoreElements() )
                }
            }
        }

        return pFrom.elements();
    }


    protected Enumeration GetToActivities( CAbstractActivity act )
    {
        Vector      pTo = new Vector();

        if( act.GetParent() != this )
            return pTo.elements();

        Enumeration flows = GetFlows();
        while( flows.hasMoreElements() )
        {
            CFlow flow = (CFlow) flows.nextElement();

            Enumeration e = flow.GetFromActivities();
            while( e.hasMoreElements() )
            {
                if( act == e.nextElement() )
                {
                    Enumeration to = flow.GetToActivities();
                    while( to.hasMoreElements() )
                        pTo.addElement( to.nextElement() );

                    break; // break -> while( e.hasMoreElements() )
                }
            }
        }

        return pTo.elements();
    }



    public void UpdateFrom( CElement elem )
    {
        if( !(elem instanceof CAbstractActivity) )
            return;

        super.UpdateFrom( elem );

        CAbstractActivity from = (CAbstractActivity) elem;
        mName           = from.mName;
        mDescription    = from.mDescription;
        mToolId         = from.mToolId;
    }


    protected CFile GetFile()
    {
        if( pParent != null )
            return pParent.GetFile();
        return null;
    }


    public String GetInfoText()
    {
        String t;

        t =  "ACTIVITY:      " + mName + "\n\n";
        //t += "STATE:         " + pState.toString()
        t += "DESCRIPTION:\n" + mDescription + "\n\n";
        t += "TYPE:          ";
        if( mToolId == 0 )
            t += "Manual (or External)";
        else
            t += "Automated";
        t += "\n\n";

        // INPUTS and OUTPUTS
        if( pParent != null )
        {
            t += "INPUTS (from activities):\n";

            boolean empty = true;
            Enumeration e = pParent.GetFromActivities( this );
            while( e.hasMoreElements() )
            {
                CAbstractActivity a = (CAbstractActivity) e.nextElement();
                t += "    " + a.GetName() + "\n";
                empty = false;
            }
            if( empty )
                    t += "    <no internal inputs for this activity>\n\n";
            else    t += "\n";



            t += "OUTPUTS (to activities):\n";

            empty = true;
            e = pParent.GetToActivities( this );
            while( e.hasMoreElements() )
            {
                CAbstractActivity a = (CAbstractActivity) e.nextElement();
                t += "    " + a.GetName() + "\n";
                empty = false;
            }
            if( empty )
                    t += "    <no internal outputs from this activity>\n\n";
            else    t += "\n";
        }

        // ROLES
        t += "ROLES:\n";
        boolean empty = true;
        Enumeration e = GetRoles();
        while( e.hasMoreElements() )
        {
            CAbstractRole role = (CAbstractRole) e.nextElement();
            t += "   " + role.GetName() + "\n";
            empty = false;
        }
        if( empty )
                t += "    <no internal roles defined to this activity>\n\n";
        else    t += "\n";


        return t;
    }

    public String toString()
    {
        return GetName();
    }

    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CAbstractActivity" );
        pStr.println( "       Name:     " + GetName() );
        pStr.println( "       DefTool:  " + GetToolId() );

        super.DEBUG_write( pStr );

        if( pSubActivities.size() > 0 )
        {
            pStr.println( "--- SubActivities and Flows ---" );

            Enumeration en = GetActivities();
            while( en.hasMoreElements() )
            {
                CAbstractActivity act = (CAbstractActivity) en.nextElement();
                act.DEBUG_write( pStr );
            }

            en = GetFlows();
            while( en.hasMoreElements() )
            {
                CFlow flow = (CFlow) en.nextElement();
                flow.DEBUG_write( pStr );
            }
            pStr.println( "---------------------" );
        }

    }


}

