/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML.CLIENT
 * Classe:  UMLCanvas
\******************************************************************************/

package TOOL.UML.CLIENT;


import  java.awt.*;
import  java.awt.event.*;

import  COPSE.*;
import  COPSE.CLIENT.*;
import  COPSE.CLIENT.GUI.*;
import  COPSE.CONTROL.*;
import  FILE.*;
import  FILE.OBJ.*;
import  GRAPH.*;
import  TOOL.UML.*;


public class    UMLCanvas
    extends     Canvas
    implements  MouseListener, MouseMotionListener
{
    protected   UMLGUIManager           pGUI;
    protected   UMLGUIToolManager       pToolManager;

    protected   CScale                  pScale = null;

    protected   CUMLFileController      pFileController = null;
    protected   CUMLGraphModel          pUMLGraphModel = null;


    // Selected Element
    protected   CUMLGraphElement        pSelectedElement;
    protected   CRectSelector           pClassSelector;
    protected   CRelationSelector       pRelationSelector;
    protected   CGraphics               pSelGraphics;

    // Creating new elements...
    protected   boolean                 mNewClass;
    protected   Rectangle               pNewClassRect;

    protected   boolean                 mNewRelation;
    protected   CPolyline               pNewRelationLine;
    protected   Point                   pNewRelationLastPoint = null;
    protected   CObjClass               pNewRelationClasses[];


    public      UMLCanvas   ( UMLGUIToolManager tool )
    {
        super();
        pToolManager = tool;
        pGUI = (UMLGUIManager)pToolManager.GetGUIManager();

        pSelectedElement = null;
        pClassSelector = null;
        pRelationSelector = null;
        pSelGraphics = null;

        mNewClass = false;
        pNewClassRect = null;
        mNewRelation = false;
        pNewRelationLine = null;
        pNewRelationClasses = new CObjClass[2];

        addMouseListener( this );
        addMouseMotionListener( this );
    }

    public CUMLGraphModel   GetUMLGraphModel()      { return pUMLGraphModel; }

    public void paint(Graphics g)
    {
        if( pUMLGraphModel != null )
        {
            CGraphics cg = new CGraphics(g, pScale);
            pUMLGraphModel.Draw( cg );

            if( pClassSelector != null )
                pClassSelector.Draw( cg );
            else
            if( pRelationSelector != null )
                pRelationSelector.Draw( cg );
        }
    }

    public void SetFileController( CFileController fileController )
    {
        pFileController = (CUMLFileController) fileController;
        if( pFileController != null )
        {
            pUMLGraphModel = new CUMLGraphModel( pFileController );
        }
        else
        {
            pUMLGraphModel = null;
            pClassSelector = null;
            pRelationSelector = null;

            mNewClass = false;
            pNewClassRect = null;
            mNewRelation = false;
            pNewRelationLine = null;
        }

        repaint();
    }


    public void AddElement( int elemId )
    {
        pUMLGraphModel.AddElement( elemId );

        repaint();
    }


    public void SelectElement( int elemId )
    {
        if( (pSelectedElement != null) &&
            (pSelectedElement.GetElementController().GetElement().GetId() == elemId) )
            return;

        CGraphics gc = new CGraphics( getGraphics() );

        // Hides old selector
        //if( (pSelectedElement!=null) && (pSelectedElement instanceof CUMLGraphClass) )
        if( pSelectedElement != null )
        {
            if( pClassSelector != null )
            {
                pClassSelector.Draw( gc );
                pClassSelector = null;
            }

            else
            if( pRelationSelector != null )
            {
                pRelationSelector.Draw( gc );
                pRelationSelector = null;
            }
        }

        pSelectedElement = pUMLGraphModel.FindElement( elemId );

        // Shows new Selector
        //if( (pSelectedElement!=null) && (pSelectedElement instanceof CUMLGraphClass) )
        if( pSelectedElement!=null )
        {
            if( pSelectedElement instanceof CUMLGraphClass )
            {
                Rectangle rect = new Rectangle( ((CUMLGraphClass)pSelectedElement).GetBounds() );
                if(pScale != null)
                    rect = pScale.RTS_Rect( rect );

                pClassSelector = new CRectSelector( new Rectangle(rect) );
                pClassSelector.Draw( gc );
            }

            else
            if( pSelectedElement instanceof CUMLGraphRelation )
            {
                CObjRelation rel = (CObjRelation) ((CUMLGraphRelation)pSelectedElement).GetElementController().GetElement();
                pRelationSelector = new CRelationSelector( rel );
                pRelationSelector.Draw( gc );
            }

        }

        pToolManager.SetSelectedElement( pSelectedElement );
    }


    public void RemoveElement( int elemId )
    {
        pUMLGraphModel.RemoveElement( elemId );

        if( (pSelectedElement != null) &&
            (pSelectedElement.GetElementController().GetElement().GetId() == elemId) )
        {
            pSelectedElement = null;
            pClassSelector = null;
            pRelationSelector = null;
        }
    }

    public void UpdateElement( int elemId )
    {
        pUMLGraphModel.UpdateElement( elemId );

        if(pSelectedElement != null)
        {
            if( pSelectedElement.GetElementController().GetElement().GetId() == elemId )
            {
                if( pSelectedElement instanceof CUMLGraphClass)
                {
                    Rectangle rect = new Rectangle( ((CUMLGraphClass)pSelectedElement).GetBounds() );
                    if(pScale != null)
                        rect = pScale.RTS_Rect( rect );
                    pClassSelector.setBounds( rect );
                }

                else
                if( pSelectedElement instanceof CUMLGraphRelation )
                {
                    // TODO: Update....
                    pRelationSelector.Update();
                }
            }
            else

            if( pSelectedElement instanceof CUMLGraphRelation )
            {
                pRelationSelector.Update();
/*
                Enumeration e = ((CObjRelation)pSelectedElement.GetElementController().GetElement()).GetExtremities();
                while( e.hasMoreElements() )
                {
                    CObjRelationExtremity extr = (CObjRelationExtremity) e.nextElement();
                    if( extr.GetClass().GetId() == elemId )
                    {
                        pRelationSelector.Update();
                        break;
                    }
                }
*/
            }
        }
    }


    public void CreateNewClass()
    {
        mNewClass = true;

        if( pSelectedElement != null )
        {
            pSelectedElement = null;
            pToolManager.SetSelectedElement( null );

            // Hide
            CGraphics g = new CGraphics( getGraphics() );
            g.setXORMode();

            if( pClassSelector != null )
            {
                pClassSelector.Draw( g );
                pClassSelector = null;
            }
            else
            if( pRelationSelector != null )
            {
                pRelationSelector.Draw( g );
                pRelationSelector = null;
            }

            g.dispose();
        }

        if( mNewRelation )
        {
            mNewRelation = false;
            pNewRelationLine = null;
        }

        setCursor( Cursor.getPredefinedCursor( Cursor.CROSSHAIR_CURSOR ) );
    }



    public void CreateNewRelation()
    {
        mNewRelation = true;

        if( pSelectedElement != null )
        {
            pSelectedElement = null;
            pToolManager.SetSelectedElement( null );

            // Hide
            CGraphics g = new CGraphics( getGraphics() );
            g.setXORMode();

            if( pClassSelector != null )
            {
                pClassSelector.Draw( g );
                pClassSelector = null;
            }
            else
            if( pRelationSelector != null )
            {
                pRelationSelector.Draw( g );
                pRelationSelector = null;
            }

            g.dispose();
        }

        if( mNewClass )
        {
            mNewClass = false;
            pNewClassRect = null;
        }

        setCursor( Cursor.getPredefinedCursor( Cursor.CROSSHAIR_CURSOR ) );
    }


    // Mouse Listener Methods
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void mousePressed(MouseEvent e)
    {
        if( pUMLGraphModel == null )
            return;

        CGraphics cg = new CGraphics( getGraphics() );
        pSelGraphics = cg.create();
        pSelGraphics.setXORMode(Color.white);


        if( e.isPopupTrigger() )
        {
            // Temporary...
            if( mNewClass || mNewRelation )
            {
                if( mNewClass )
                {
                    mNewClass = false;
                    pSelGraphics.drawRect( pNewClassRect.x, pNewClassRect.y, pNewClassRect.width, pNewClassRect.height );
                    pNewClassRect = null;
                }
                else
                {
                    mNewRelation = false;
                    pSelGraphics.drawPolyline( pNewRelationLine );
                    pNewRelationLine = null;
                }

                setCursor( Cursor.getDefaultCursor() );
            }
            return;
        }


        Point p = e.getPoint();
        if( pScale != null )
            p = pScale.STR_Point(p);


        if( (pClassSelector != null) && pClassSelector.Contains(p.x,p.y) )
        {
            pClassSelector.MouseDown( pSelGraphics, p.x, p.y );
            setCursor( pClassSelector.GetCursor(p.x, p.y) );
            return;
        }

        if( (pRelationSelector != null) && pRelationSelector.Contains(p.x,p.y) )
        {
            pRelationSelector.MouseDown( pSelGraphics, p.x, p.y );
            setCursor( pRelationSelector.GetCursor(p.x, p.y) );
            return;
        }


        CUMLGraphElement newSelection = pUMLGraphModel.GetElementAt( p.x, p.y );


        // IF Creating New Class
        if( mNewClass )
            if( newSelection == null )
            {
                pNewClassRect = new Rectangle( p );
                return;
            }
            else
            {   // Cancel "New Class" and continue... (select the object)
                mNewClass = false;
                setCursor( Cursor.getDefaultCursor() );
            }


        // IF Creating New Relation
        if( mNewRelation )
        {
            if( (pNewRelationLine != null) && (p.equals( pNewRelationLine.GetLastPoint() )) )
            {   // cancel...
                mNewRelation = false;
                pNewRelationLine = null;
                pNewRelationLastPoint = null;

                setCursor( Cursor.getDefaultCursor() );
                return;
            }

            if( newSelection != null )
            {
                if( pNewRelationLine == null )
                {
                    if( newSelection instanceof CUMLGraphClass )
                    {
                        // Start relation...
                        pNewRelationLine = new CPolyline();
                        pNewRelationLine.AddPoint( p );
                        pNewRelationLastPoint = p;
                        pNewRelationClasses[0] = (CObjClass) newSelection.GetElementController().GetElement();
                        return;
                    }
                    else
                    {
                        // Cancel...
                        setCursor( Cursor.getDefaultCursor() );
                        mNewRelation = false;
                    }
                }
                else
                {
                    if( newSelection instanceof CUMLGraphClass )
                    {
                        // End relation...
                        setCursor( Cursor.getDefaultCursor() );
                        pNewRelationLine.AddPoint( p );
                        pNewRelationLastPoint = p;
                        pNewRelationClasses[1] = (CObjClass) newSelection.GetElementController().GetElement();
                        pToolManager.CreateNewRelation( pNewRelationLine, pNewRelationClasses );

                        mNewRelation = false;
                        pNewRelationLine = null;
                        pNewRelationLastPoint = null;
                        return;
                    }
                    else
                    {
                        // Add Point...
                        pNewRelationLine.AddPoint( p );
                        pNewRelationLastPoint = p;
                        return;
                    }
                }
            }
            else
            {
                // new selection == null
                if( pNewRelationLine == null )
                {
                    // Cancel...
                    mNewRelation = false;
                    setCursor( Cursor.getDefaultCursor() );
                }
                else
                {
                    // Add Point...
                    pNewRelationLine.AddPoint( p );
                    pNewRelationLastPoint = p;
                    return;
                }
            }

        } // end IF mNewRelation...


        // Selecting an element
        if( newSelection != pSelectedElement )
        {
            Rectangle rect;

            // Hides old selector
            if( pSelectedElement!=null )
            {
                if( pClassSelector != null )
                {
                    pClassSelector.Draw( pSelGraphics );
                    pClassSelector = null;
                }

                else
                if( pRelationSelector != null )
                {
                    pRelationSelector.Draw( pSelGraphics );
                    pRelationSelector = null;
                }
            }

            // Set new Element selected
            pSelectedElement = newSelection;


            // Shows new Selector
            if( pSelectedElement!=null )
            {
                if( pSelectedElement instanceof CUMLGraphClass )
                {
                    rect = new Rectangle( ((CUMLGraphClass)pSelectedElement).GetBounds() );
                    if(pScale != null)
                        rect = pScale.RTS_Rect( rect );

                    pClassSelector = new CRectSelector( new Rectangle(rect) );
                    pClassSelector.MouseDown( pSelGraphics, p.x, p.y );
                    pClassSelector.Draw( pSelGraphics );
                }

                else
                if( pSelectedElement instanceof CUMLGraphRelation )
                {
                    CObjRelation rel = (CObjRelation) ((CUMLGraphRelation)pSelectedElement).GetElementController().GetElement();
                    pRelationSelector = new CRelationSelector( rel );
                    pRelationSelector.MouseDown( pSelGraphics, p.x, p.y );
                    pRelationSelector.Draw( pSelGraphics );
                }

            }
        }

        pToolManager.SetSelectedElement( pSelectedElement );
    }


    public void mouseReleased(MouseEvent e)
    {
        Point p = e.getPoint();
        if( pScale != null )
            p = pScale.STR_Point( p );

        if( pClassSelector!=null )
        {
            //CGraphics cg = new CGraphics( getGraphics() );
            //cg.setXORMode(Color.white);
            if( pClassSelector.IsMouseDown() )
            {
                pClassSelector.MouseUp( pSelGraphics, p.x, p.y );
                //pSelGraphics = null;

                if( pClassSelector.HasMoved() )
                {
                    Rectangle r = pClassSelector.getBounds();
                    CElement element = pSelectedElement.GetElementController().GetElement();

                    try{
                        CEvent event = new CEvent( CEvent.EvUMLEditor_Class_Position );
                        event.AddArgument( new Integer( element.GetId() ) );
                        event.AddArgument( r );
                        pGUI.Send(event);
                    } catch(Exception exc) {}
                }
            }
        }
        else

        if( pRelationSelector!=null )
        {
            if( pRelationSelector.IsMouseDown() )
            {
                pRelationSelector.MouseUp( pSelGraphics, p.x, p.y );

                if( pRelationSelector.HasMoved() )
                {
                    /* */
                    // Internal Points...
                    CPolyline poly = new CPolyline( pRelationSelector.GetPolyline() );
                    poly.RemovePoint( poly.GetFirstPoint() );
                    poly.RemovePoint( poly.GetLastPoint() );

                    CElement element = pSelectedElement.GetElementController().GetElement();

                    try{
                        CEvent event = new CEvent( CEvent.EvUMLEditor_Relation_Position );
                        event.AddArgument( new Integer( element.GetId() ) );
                        event.AddArgument( pRelationSelector.GetExtremitySidePosition(0) );
                        event.AddArgument( pRelationSelector.GetExtremitySidePosition(1) );
                        event.AddArgument( poly );
                        pGUI.Send(event);
                    } catch(Exception exc) {}
                    /* */
                }
            }
        }
        else

        if( mNewClass )
        {
            Dimension size = new Dimension( p.x - pNewClassRect.x, p.y - pNewClassRect.y );
            pNewClassRect.setSize( size );
            pToolManager.CreateNewClass( pNewClassRect );

            mNewClass = false;
            setCursor( Cursor.getDefaultCursor() );
        }

        else

        if( mNewRelation )
        {
            // Don't dispose the pSelGraphics...
            return;
        }

        if( pSelGraphics != null )
        {
            pSelGraphics.dispose();
            pSelGraphics = null;
        }


    }




    // Mouse Motion Listener Methods
    public void mouseDragged(MouseEvent e)
    {
        Point p = e.getPoint();
        if( pScale != null )
            p = pScale.STR_Point( p );

        if( pClassSelector != null )
        {
            if( pClassSelector.IsMouseDown() )
            {
                pClassSelector.MouseMove( pSelGraphics, p.x, p.y );
            }
        }
        if( pRelationSelector != null )
        {
            if( pRelationSelector.IsMouseDown() )
            {
                pRelationSelector.MouseMove( pSelGraphics, p.x, p.y );
            }
        }

        else
        if( mNewClass )
        {
            Dimension size = new Dimension( p.x - pNewClassRect.x, p.y - pNewClassRect.y );
            pSelGraphics.drawRect( pNewClassRect.x, pNewClassRect.y, pNewClassRect.width, pNewClassRect.height );
            pNewClassRect.setSize( size );
            pSelGraphics.drawRect( pNewClassRect.x, pNewClassRect.y, pNewClassRect.width, pNewClassRect.height );
        }

        else
        if( mNewRelation )
        {
            if( (pNewRelationLine != null) && (pSelGraphics!=null) )
            {
                Point last = pNewRelationLine.GetLastPoint();
                pSelGraphics.drawLine( last.x, last.y, pNewRelationLastPoint.x, pNewRelationLastPoint.y );
                pNewRelationLastPoint = p;
                pSelGraphics.drawLine( last.x, last.y, pNewRelationLastPoint.x, pNewRelationLastPoint.y );
            }

        }
    }


    public void mouseMoved(MouseEvent e)
    {
        Point p = e.getPoint();
        if( pScale != null )
            p = pScale.STR_Point( p );

        if( pClassSelector != null )
        {
            Cursor cursor = pClassSelector.GetCursor( p.x, p.y );
            if( cursor != getCursor() )
                setCursor( cursor );
        }

        else
        if( pRelationSelector != null )
        {
            Cursor cursor = pRelationSelector.GetCursor( p.x, p.y );
            if( cursor != getCursor() )
                setCursor( cursor );
        }

        else
        if( mNewRelation )
        {
            if( (pNewRelationLine != null) && (pSelGraphics!=null) )
            {
                Point last = pNewRelationLine.GetLastPoint();
                pSelGraphics.drawLine( last.x, last.y, pNewRelationLastPoint.x, pNewRelationLastPoint.y );
                pNewRelationLastPoint = p;
                pSelGraphics.drawLine( last.x, last.y, pNewRelationLastPoint.x, pNewRelationLastPoint.y );
            }
        }

        /* * /
        else
        {
            Cursor defaultCursor = Cursor.getDefaultCursor();
            if( getCursor() != defaultCursor )
                setCursor( defaultCursor );
        }
        /* */
    }


}
