/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML
 * Classe:  CUMLGraphRelation
\******************************************************************************/

package TOOL.UML;


import  java.awt.*;
import  java.util.*;

import  FILE.*;
import  FILE.OBJ.*;

import  GRAPH.*;

import  COPSE.CONTROL.*;
import  COPSE.CUserSession;

/*************************************************************************
 *  Class CUMLGraphRelation
 *************************************************************************/
public class CUMLGraphRelation
    extends CUMLGraphElement
{
    // Data
    protected   CObjRelation    pRelation;


    // Presentation
    //protected   Rectangle       pBase;
    protected   CGrpObject      pgSelected;


    protected   CGrpPolyline    pgLine;         // Relation Line

    protected   CGrpLabel       pgCard1,        // Extrem 1 Cardinality
                                pgCard2;        // Extrem 2 Cardinality

    protected   CUMLGraphLabel  pgLabel1,       // Extrem 1 Label
                                pgLabel2;       // Extrem 2 Label

    protected   CGrpObject      pgRelType;      // Lozenge  => Aggregation
                                                // Arrow    => Instanciation / Association (Navegability)
                                                // Triangle => Inheritance

    public CUMLGraphRelation ( CVisualElementController elemController )
    {
        super( elemController );
        pRelation = (CObjRelation) elemController.GetElement();

        _CreateGraphicObjects();
    }


    private Rectangle _CalculateRect( int side, Point p, int w, int h )
    {
        int x, y;

        switch( side )
        {
            case CObjRelationExtremity.ExtTop:
                x = p.x - w/2;
                y = p.y - h;
                p.y -= h;
                break;

            case CObjRelationExtremity.ExtBottom:
                x = p.x - w/2;
                y = p.y;
                p.y += h;
                break;

            case CObjRelationExtremity.ExtLeft:
                x = p.x - w;
                y = p.y - h/2;
                p.x -= w;
                break;

            default:
            //case CObjRelationExtremity.ExtRight:
                x = p.x;
                y = p.y - h/2;
                p.x += w;
        }

        return( new Rectangle(x,y,w,h) );
    }




    private void _CreateGraphicObjects()
    {
        // Relationship Line
        CPolyline poly = new CPolyline();
        Enumeration points = pRelation.GetPoints().elements();
        while( points.hasMoreElements() )
            poly.AddPoint( (Point) points.nextElement() );

        pgLine = new CGrpPolyline( poly );


        // Relation Type
        pgRelType = null;

        int type = pRelation.GetType();
        CObjRelationExtremity extr = pRelation.GetExtremity(0);
        int side = extr.GetClassSide();
        int direction;
        switch( side )
        {
            case CObjRelationExtremity.ExtTop:      direction = CGrpTriangle.DOWN;      break;
            case CObjRelationExtremity.ExtBottom:   direction = CGrpTriangle.UP;        break;
            case CObjRelationExtremity.ExtLeft:     direction = CGrpTriangle.RIGHT;     break;
            case CObjRelationExtremity.ExtRight:
            default:                                direction = CGrpTriangle.LEFT;
        }

        Point p = poly.GetFirstPoint();

        switch( type )
        {
            case CObjRelation.Association:
                // Verify Navegability...
                {
                    p = null;
                    if( extr.HasArrow() )
                        p = poly.GetFirstPoint();
                    else
                    {
                        extr = pRelation.GetExtremity(1);
                        if( extr.HasArrow() )
                        {
                            p = poly.GetLastPoint();
                            side = extr.GetClassSide();
                        }
                    }

                    if( p != null )
                    {
                        switch( side )
                        {
                            case CObjRelationExtremity.ExtTop:
                                pgRelType = new CGrpArrow( _CalculateRect( side, p, 10, 10 ), CGrpArrow.DOWN );
                                break;

                            case CObjRelationExtremity.ExtBottom:
                                pgRelType = new CGrpArrow( _CalculateRect( side, p, 10, 10 ), CGrpArrow.UP );
                                break;

                            case CObjRelationExtremity.ExtLeft:
                                pgRelType = new CGrpArrow( _CalculateRect( side, p, 10, 10 ), CGrpArrow.RIGHT );
                                break;

                            case CObjRelationExtremity.ExtRight:
                            default:
                                pgRelType = new CGrpArrow( _CalculateRect( side, p, 10, 10 ), CGrpArrow.LEFT );
                        }
                    }
                } // Association
                break;


            case CObjRelation.Inheritance:
                {
                    pgRelType = new CGrpTriangle( _CalculateRect( side, p, 12, 12 ), direction );
                } // Inheritance
                break;


            case CObjRelation.AggregationByRef:
            case CObjRelation.AggregationByValue:
                {
                    switch( side )
                    {
                        case CObjRelationExtremity.ExtTop:
                        case CObjRelationExtremity.ExtBottom:
                            pgRelType = new CGrpLozenge( _CalculateRect( side, p, 12, 18 ) );
                            break;
                        default:
                        //case CObjRelationExtremity.ExtLeft:
                        //case CObjRelationExtremity.ExtRight:
                            pgRelType = new CGrpLozenge( _CalculateRect( side, p, 18, 12 ) );
                    }

                    if( type == CObjRelation.AggregationByValue )
                    {
                        pgRelType.SetFilled( true );
                        //pgRelType.SetFillColor( Color.black );
                    }
                } // Aggregation
                break;


            case CObjRelation.Instanciation:
                {
                    pgRelType = new CGrpArrow( _CalculateRect( side, p, 10, 10 ), direction );
                    pgLine.SetDashed( true );
                } // Instanciation
                break;

        } // switch( type ) => Relation Type...


        // Cardinalities and Labels
        CObjCardinality  card;

        extr = pRelation.GetExtremity(0);
        card = extr.GetCardinality();
        pgCard1 = new CGrpLabel( card.GetPoint(), card.GetText(), card.GetAlign(), null );
        pgLabel1 = new CUMLGraphLabel( extr.GetLabel() );

        extr = pRelation.GetExtremity(1);
        card = extr.GetCardinality();
        pgCard2 = new CGrpLabel( card.GetPoint(), card.GetText(), card.GetAlign(), null );
        pgLabel2 = new CUMLGraphLabel( extr.GetLabel() );


    }// _CreateGraphicObjects


    public void Update()
    {
        _CreateGraphicObjects();
    }

    public void Draw (CGraphics graf)
    {
        // Context: g1 (without clipping)
        CGraphics g1 = graf.create();


        if( pElemController.IsLocked() )
        {
        	// temporary
            int connId = pElemController.GetLockerConnectionId();
            //int userId =
        	Color userColor = CUserSession.getUserColor( connId );
            g1.setColor( userColor );
        }


        // Line
        pgLine.Draw( g1 );

        if( pgRelType != null )
        {
            if( pgRelType.IsFilled() )
                pgRelType.SetFillColor( g1.getColor() );
            pgRelType.Draw( g1 );
        }

        int visual = pElemController.GetVisual();
        if( visual != 0 ) // MINIMUM
        {
            // If it is nor minimum ... (Partial or Total)
            //Font oldFont = g1.getFont();
            g1.setFont( new Font( "Helvetica", Font.PLAIN, 10 ) );

            // Cardinalidades
            pgCard1.Draw( g1 );
            pgCard2.Draw( g1 );


            if( visual == 2 ) // Total
            {
                //Labels
                pgLabel1.Draw( g1 );
                pgLabel2.Draw( g1 );
            }

            //g1.setFont( oldFont );
        }

        g1.dispose();
    }


    public boolean Contains( int x, int y )
    {
        if( pgLine.Contains(x,y) )
        {
            pgSelected = pgLine;
            return true;
        }

        if( pgRelType != null )
            if( pgRelType.Contains(x,y) )
            {
                pgSelected = pgRelType;
                return true;
            }

        if( pgCard1.Contains(x,y) )
        {
            pgSelected = pgCard1;
            return true;
        }

        if( pgCard2.Contains(x,y) )
        {
            pgSelected = pgCard2;
            return true;
        }

        if( pgLabel1.Contains(x,y) )
        {
            pgSelected = pgLabel1;
            return true;
        }

        if( pgLabel2.Contains(x,y) )
        {
            pgSelected = pgLabel2;
            return true;
        }

        return false;
    }



}


