/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: COPSEServerEventHandler
\******************************************************************************/


import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import COPSE.CONTROL.*;

public class COPSEServerEventHandler
    extends CEventHandler
{
    protected   COPSEServer     pServer;

    public COPSEServerEventHandler ( COPSEServer server )
    {
        super();
        pServer = server;
    }


    protected void ProcessEvent( CEvent event, CConnectionHandler handler )
    {
        COPSEServerConnectionHandler pHandler = (COPSEServerConnectionHandler) handler;
        COPSEServerSessionManager    pSessionManager = pServer.GetSessionManager();

        int type = event.GetType();
        System.out.println( "[DEBUG] COPSEServerEventHandler: Event ["+type+"]" );

        switch( type )
        {
            case CEvent.EvCOPSE_Connection_Close:
                {
                    COPSEServerSession session = pHandler.GetSession();
                    int userId = session.GetUserId();


                    // Notify all clients
                    event = new CEvent( CEvent.EvCOPSE_Connection_Close );
                    event.AddArgument( new Integer(userId) );
                    session.Notify( event );

                    // Remove session from SessionManager
                    pSessionManager.RemoveSession( session );
                    pSessionManager.NotifyAll( event );

                    // Close connection
                    session.Close();

                    System.out.println( "[DEBUG} COPSEServerEventHandler - User ["+userId+"] logout" );
                    break;
                }

            case CEvent.EvCOPSE_SessionManager_Load:
                {
                    COPSESessionManager sMgr = pSessionManager.CloneAsCOPSESessionManager();

                    event = new CEvent( CEvent.EvCOPSE_SessionManager_Load );
                    event.AddArgument( sMgr );
                    try {
                        pHandler.Send( event );
                    } catch(IOException e){};

                    break;
                }

            case CEvent.EvCOPSE_ProductManager_Load:
                {
                    COPSEServerProductManager productManager;
                    CFileController     productController;
                    Vector              documentControllers;

                    productManager      = pServer.GetProjectManager().GetProductManager();
                    productController   = productManager.GetProductController();
                    documentControllers = productManager.GetDocumentControllers();

                    event = new CEvent( type );
                    event.AddArgument( productController );
                    event.AddArgument( documentControllers );
                    try {
                        pHandler.Send( event );
                    } catch(IOException e){};

                    break;
                }

            case CEvent.EvCOPSE_UsersManager_Load:
                {
                    COPSEServerUsersManager usersManager;
                    CFileController     usersController;

                    usersManager      = pServer.GetProjectManager().GetUsersManager();
                    usersController   = usersManager.GetUsersController();

                    event = new CEvent( type );
                    event.AddArgument( usersController );
                    try {
                        pHandler.Send( event );
                    } catch(IOException e){};

                    break;
                }


            case CEvent.EvCOPSE_ProcessManager_ServerSocket:
                {
                    COPSEServerProcessManager pProcessManager = pServer.GetProjectManager().GetProcessManager();
                    String   host = pProcessManager.GetServerSocketHost();
                    int      port = pProcessManager.GetServerSocketPort();

                    event = new CEvent( type );
                    event.AddArgument( host );
                    event.AddArgument( new Integer(port) );
                    try {
                        pHandler.Send( event );
                    } catch(IOException e){};
                }
                break;


            default:

        }


        /* * /
        if( type <= CEvent.EvCOPSE_Connection_Exit )
            ProcessEvent_Session( event );
        else
        if( (type > CEvent.Ev...) && (type <= CEvent.Ev...) )
            ProcessEvent_Product( event );
        else
            ProcessEvent_Default( event );
        /* */
    }

}