/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: COPSEServerProcessManager
\******************************************************************************/


import java.io.*;
import java.util.*;

import COPSE.*;
import COPSE.CONTROL.*;

import FILE.*;
import PROCESS.*;
import PROCESS.MODEL.*;


// Event Handlers
import COPSE.SERVER.*;


class COPSEServerProcessServerEventHandler
    extends CServerEventHandler
{
    public COPSEServerProcessServerEventHandler( COPSEServerProcessManager manager )
    {
        super( manager );
    }
}


class COPSEServerProcessManagerEventHandler
    extends CServerManagerEventHandler
{
    public COPSEServerProcessManagerEventHandler( COPSEServerProcessManager manager, CServerEventHandler handler )
    {
        super( manager, handler );
    }
}




public class COPSEServerProcessManager
    extends CQueuedServerManager
//    extends CServerManager
{
    COPSEServerProjectManager   pProjectManager;

    CProcess                    pPrjProcess;
    CProcessUsersSession        pPrjUsersSession;


//    protected static int GetToolId

    public static COPSEServerProcessManager New (COPSEServerProjectManager projectManager)
        throws Exception
    {
        COPSEServerProcessManager manager;
        CProcess process;

        try{
            process = (CProcess) CFile.Load( "process.copse", projectManager.GetProjectDir() );
        } catch (Exception e )
        {
            process = _CreateDefaultProcess( projectManager.GetProjectDir() );
        }

        manager = new COPSEServerProcessManager( projectManager, process );
        manager.start();

        return manager;
    }



    public COPSEServerProcessManager( COPSEServerProjectManager manager, CProcess process )
        throws Exception
    {
        super( manager.GetNextServerSocketPort(), new CLocator(0,0,0), process );
        pProjectManager = manager;

        pPrjUsersSession = new CProcessUsersSession();
    }



    private static CProcess _CreateDefaultProcess( String projectDir )
    {
        CProcess process = new CProcess( "process.copse", projectDir );

        CModelHeader header;
        process.AddElement( header = new CModelHeader() );
        process.SetModelHeader( header );
        header.SetName( "Process" );

        CSubModel   submodel;
        process.AddElement( submodel = new CSubModel() );

        header.SetSubModel( submodel );

        process.SetupActivities();

        try {
            process.Save();
        } catch (IOException e) {};
        return process;
    }


    protected CServerEventHandler   CreateServerEventHandler()
    {
        return new COPSEServerProcessServerEventHandler ( this );
    }

    protected CServerManagerEventHandler CreateManagerEventHandler()
    {
        return new COPSEServerProcessManagerEventHandler( this, pEventHandler );
    }


    protected CFileController CreateFileController()
    {
        // Create the default FileController
        pPrjProcess = (CProcess) pFile;
        //return new CProcessController( pFile );
        return new CFileController( pFile );
    }



//***********************************************************************

    public CProcess GetProcess()
    {
        return pPrjProcess;
    }



    public void AddUserToActivity( int actId, int userId )
    {
        pPrjUsersSession.AddEntry( actId, userId );
    }

    public void RemoveUserFromActivity( int actId, int userId )
    {
        pPrjUsersSession.RemoveEntry( actId, userId );
    }

    public Enumeration GetUsersInActivity( int actId )
    {
        return pPrjUsersSession.GetUsersInActivity( actId );
    }

    public CProcessUsersSession GetProcessUsersSession()
    {
        return pPrjUsersSession;
    }



}


