/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: UserManager
\******************************************************************************/


import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import FILE.*;
import AGENT.*;


public class UserManager
    extends Thread
{
    private     COPSEManager    pManager;

    private     ServerSocket    pServerSocket;

    private     Vector          pLoggedUsers;
    private     CFile           pPrjUsers;

    private     boolean         mExit;  // If "Exit" was requested


    public  UserManager ( COPSEManager manager, String serverSocketPort )
        throws IOException
    {
        super();

        // The exception will be thrown to the upper object
        //try {
        pServerSocket = new ServerSocket( new Integer(serverSocketPort).intValue(), 8 );
        //} catch( java.io.IOException e1 ) {}


        // The Server Socket was succesfully binded.
        // Then, the remainer UserManager attributes can be initialized
        pManager = manager;
        pLoggedUsers = new Vector();
        pPrjUsers = null;
        mExit = false;
    }


    public String GetHostName()
    {
        try {  return pServerSocket.getInetAddress().getHostName(); }
        catch( Exception e ) { return ""; }
    }

    public int GetSocketPort()
    {
        return pServerSocket.getLocalPort();
    }

    public void SetProjectUsers( CFile prjUsers )
    {
        pPrjUsers = prjUsers;
    }


    public CFile GetAllUsers()
    {
        return pPrjUsers;
    }

    public Vector GetLoggedUsersIds ()
    {
        Vector usersIds = new Vector();

        Enumeration e = pLoggedUsers.elements();
        while( e.hasMoreElements() )
        {
            UserServer us = (UserServer) e.nextElement();
            usersIds.addElement( new Integer( us.GetUserId() ) );
        }

        return usersIds;
    }


    public void run()
    {
        // Accepting new connections from User Clients
        Socket slaveSocket = null;

        try {
            pServerSocket.setSoTimeout( 1 );
        } catch( SocketException e ){}

        mExit = false;
        while( !mExit )
        {
            slaveSocket = null;
            // Wait for next "open user connection" request
            try {
                slaveSocket = pServerSocket.accept();
            }
            catch( java.io.IOException e2 ) {}
            catch( Exception ex ) {}
            finally
            {
                if( slaveSocket != null )
                {
                    // Try to open connection after verifying the user login
                    System.out.println( "DEBUG: User Manager - Connecting User ..." );
                    _ConnectUser( slaveSocket );
                }
            }
        }


    }


    private void _ConnectUser( Socket socket )
    {
        ObjectInputStream  input;
        ObjectOutputStream output;

        boolean     ok = false;
        String      msg = "";

        int         userId = 0;

        try {
            System.out.println( "DEBUG: User Manager - Getting Streams ..." );
            InputStream in = socket.getInputStream();
            OutputStream out = socket.getOutputStream();

            System.out.println( "DEBUG: User Manager - Creating Object Streams ..." );
            output = new ObjectOutputStream( out );
            input = new ObjectInputStream( in );

            System.out.println( "DEBUG: User Manager - Loading Connection Command ..." );
            CEvent event = (CEvent)input.readObject();

            if( event.GetType() != CEvent.EvUserConnection_Open )
            {
                msg = "Invalid initial connection command!";
            }
            else
            {
                String login    = (String) event.GetArgumentAt(0);
                String password = (String) event.GetArgumentAt(1);

                // TODO: Verify login and password of user
                CMember member = _FindUserByLogin( login );
                if( (member != null) && (member.CheckPassword(password)) )
                {
                    System.out.println( "DEBUG: User Manager - User '"+login+"' logged..." );
                    ok = true;
                    userId = member.GetId();
                }
                else
                {
                    if( member == null )
                         msg = "Invalid login!";
                    else msg = "Invalid Password!";
                    System.out.println( "DEBUG: User Manager - "+msg );
                }


                /* / Temporary
                if( login.equals( password ) )
                {
                    System.out.println( "DEBUG: User Manager - User '"+login+"' logged..." );
                    ok = true;
                    userId = pLoggedUsers.size() + 1;
                }
                else
                {
                    System.out.println( "DEBUG: User Manager - Invalid Password for '"+login+"'" );
                    msg = "Invalid Password";
                }
                /// End: Temporary
                */
            }

            if( ok )
            {
                synchronized(this)
                {
                    System.out.println( "DEBUG: UserManager - Creating UserServer ...");
                    UserServer userServer = new UserServer( this, userId, socket, input, output );
                    pLoggedUsers.addElement( userServer );
                    System.out.println( "DEBUG: UserManager - Starting UserServer ...");
                    userServer.start();

                    event = new CEvent( CEvent.EvUserConnection_Open );
                    event.AddArgument( new Integer( userId ) );
                    NotifyUsers( event );
                }
            }
            else
            {
                event = new CEvent( CEvent.EvUserConnection_Error );
                event.AddArgument( msg );
                output.writeObject( event );
            }

        } catch ( Exception e ) {}
    }

    private void _CloseAllConnections()
    {
        CEvent event = new CEvent( CEvent.EvUserConnection_Exit );

        Enumeration e = pLoggedUsers.elements();
        while( e.hasMoreElements() )
        {
            UserServer userServer = (UserServer) e.nextElement();
            userServer.Send( event );
            userServer.Exit();
        }

        e = pLoggedUsers.elements();
        while( e.hasMoreElements() )
        {
            UserServer userServer = (UserServer) e.nextElement();
            try {
                userServer.join( 100 );
            } catch( InterruptedException ie ){};
            if( userServer.isAlive() )
                userServer.stop();
        }

        pLoggedUsers.removeAllElements();
    }

    private CMember _FindUserByLogin( String login )
    {
        Enumeration e = pPrjUsers.GetElements();
        while( e.hasMoreElements() )
        {
            CElement elem = (CElement) e.nextElement();

            if( elem instanceof CMember )
            {
                CMember mem = (CMember) elem;
                if( mem.GetLogin().equals(login) )
                    return mem;
            }
        }
        return null;
    }

    public synchronized void RemoveUserConnection( UserServer server )
    {
        if( Thread.currentThread() == server )
        {
            pLoggedUsers.removeElement(server);

            CEvent event = new CEvent( CEvent.EvUserConnection_Close );
            event.AddArgument( new Integer(server.GetUserId()) );
            NotifyUsers( event );
        }
        else
            System.out.println( "ERROR: User Manager - Remove User Connection ...");
    }


    public synchronized void NotifyUser ( int userId, CEvent event )
    {
        Enumeration e = pLoggedUsers.elements();
        while( e.hasMoreElements() )
        {
            UserServer userServer = (UserServer) e.nextElement();
            if( userServer.GetUserId() == userId )
            {
                userServer.Send( event );
                return;
            }
        }
    }

    public synchronized void NotifyUsers ( CEvent event )
    {
        System.out.println( "DEBUG: UserManager - Notifying Users About Event '"+event.GetType()+"'");
        System.out.println( "DEBUG: UserManager - There is(are) "+ pLoggedUsers.size() +" UserServer(s)");

        Enumeration e = pLoggedUsers.elements();
        while( e.hasMoreElements() )
        {
            UserServer userServer = (UserServer) e.nextElement();
            userServer.Send( event );
        }
    }

    public void Exit()
    {
        mExit = true;

        synchronized( this )
        {
            System.out.println( "DEBUG: closing all connections...");
            _CloseAllConnections();

            System.out.println( "DEBUG: closing ServerSocket...");
            try {
                pServerSocket.close();
            } catch( IOException e ) {};
        }

    }

}

