/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: UserServer
\******************************************************************************/


import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;


public class UserServer
    extends Thread
{
    UserManager         pUserManager;

    int                 mUserId;
    Socket              pSocket;
    ObjectInputStream   pInput;
    ObjectOutputStream  pOutput;

    boolean             mExit;  // If an Exit request has been done


    public UserServer ( UserManager uMgr, int uId, Socket sock,
        ObjectInputStream input, ObjectOutputStream output )
    {
        super();

        pUserManager = uMgr;
        mUserId = uId;
        pSocket = sock;
        pInput = input;
        pOutput = output;

        mExit = false;
    }


    public void run()
    {
        while( !mExit )
        {
            try
            {
                System.err.println( "DEBUG: Receiving Command ...");
                CEvent event = (CEvent) pInput.readObject();
                System.err.println( "DEBUG: Processing Command ...");
                _ProcessUserRequest( event );
            }
            catch( ClassCastException e1 ) {}
            catch( ClassNotFoundException e2 ) {}
            catch( IOException e3 )
            {
                // TODO: Verify whether the socket is ok.
                //       if it is ok, continue
                //       else, "mExit = true;" => Connection has fallen !!
            }
        }

        // ??? ...
        try{
            pSocket.close();
        } catch( IOException ioe ){};
    }


    public int GetUserId()      { return mUserId; }


    private void _ProcessUserRequest (CEvent event)
    {
        switch( event.GetType() )
        {
            case CEvent.EvUserManager_Users_LoadAll:
                {
                    CEvent answer = new CEvent( event.GetType() );
                    answer.AddArgument( pUserManager.GetAllUsers() );
                    Send( answer );
                }
                break;

            case CEvent.EvUserManager_Users_LoadLogged:
                {
                    CEvent answer = new CEvent( event.GetType() );
                    answer.AddArgument( pUserManager.GetAllUsers() );
                    Send( answer );
                }
                break;

            case CEvent.EvUserConnection_Close:
                {
                    System.err.println( "DEBUG: Received Close Command ...");

                    CEvent answer = new CEvent( CEvent.EvUserConnection_Close );
                    answer.AddArgument( new Integer(mUserId) );
                    System.err.println( "DEBUG: Sending Close Confirmation ...");
                    Send( answer );

                    mExit = true;
                    pUserManager.RemoveUserConnection( this );
                }
                break;

            default:
                {
                    CEvent answer = new CEvent( CEvent.EvUserConnection_Error );
                    answer.AddArgument( new String( "Unknown event #("+ event.GetType()+") !") );
                    Send( answer );
                }
        }
    }

    public synchronized void Send (CEvent event)
    {
        try {
            pOutput.writeObject( event );
        } catch( Exception e ) {};
    }

    public void Exit()
    {
        try{
            pSocket.close();
        } catch(IOException e) {}
        stop();
    }
}